@@ -34,7 +34,6 @@ using System.IO;
 using System.Xml;
 using System.Collections.Generic;
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Projects;
@@ -67,7 +66,7 @@ namespace MonoDevelop.AspNet
 		[ItemProperty ("Target", ValueType=typeof(WebDeployTarget), Scope="*")]
 		protected WebDeployTargetCollection webDeployTargets = new WebDeployTargetCollection ();
 		
-		ProjectRegisteredControls controlRegistrationCache;
+		RegistrationCache registrationCache;
 		CodeBehindTypeNameCache codebehindTypeNameCache;
 		
 		#region properties
@@ -121,37 +120,18 @@ namespace MonoDevelop.AspNet
 			get { return webDeployTargets; }
 		}
 		
-		internal ProjectRegisteredControls ControlRegistrationCache {
+		internal RegistrationCache RegistrationCache {
 			get {
-				if (controlRegistrationCache == null)
-					controlRegistrationCache = new ProjectRegisteredControls (this);
-				
-				return controlRegistrationCache;
+				if (registrationCache == null)
+					registrationCache = new RegistrationCache (this);
+				return registrationCache;
 			}
 		}
 		
-		public override ClrVersion[] SupportedClrVersions {
-			get {
-				ClrVersion[] versions = base.SupportedClrVersions;
-				if (versions == null)
-					return null;
-				
-				bool ver1 = false, ver2 = false;
-				foreach (ClrVersion version in versions) {
-					if (version == ClrVersion.Net_1_1)
-						ver1 = true;
-					if (version == ClrVersion.Net_2_0)
-						ver2 = true;
-				}
-				if (ver1) {
-					if (ver2)
-						return new ClrVersion[] { ClrVersion.Net_1_1, ClrVersion.Net_2_0 };
-					else return new ClrVersion[] { ClrVersion.Net_1_1 };
-				} else if (ver2) {
-					return new ClrVersion[] { ClrVersion.Net_2_0 };
-				}
-				return null;
-			}
+		public override bool SupportsFramework (TargetFramework framework)
+		{
+			//only support 1.1, 2.0, 3.5, 4.0 etc, not monotouch, moonlight and so on
+			return framework.IsCompatibleWithFramework ("1.1") && base.SupportsFramework (framework);
 		}
 		
 		#endregion
@@ -172,18 +152,18 @@ namespace MonoDevelop.AspNet
 		public AspNetAppProject (string languageName, ProjectCreateInformation info, XmlElement projectOptions)
 			: base (languageName, info, projectOptions)
 		{
-			foreach (AspNetAppProjectConfiguration conf in Configurations)
-				conf.OutputDirectory = "bin";
 			Init ();
+			
+			var binPath = info == null? (FilePath)"bin" : info.BinPath;
+			foreach (AspNetAppProjectConfiguration cfg in Configurations)
+				cfg.OutputDirectory = binPath;
 		}	
 		
 		public override SolutionItemConfiguration CreateConfiguration (string name)
 		{
-			AspNetAppProjectConfiguration conf = new AspNetAppProjectConfiguration ();
-			conf.Name = name;
-			conf.OutputDirectory = String.IsNullOrEmpty (BaseDirectory)? "bin" : Path.Combine (BaseDirectory, "bin");
-			if (LanguageBinding != null)
-				conf.CompilationParameters = LanguageBinding.CreateCompilationParameters (null);			
+			var conf = new AspNetAppProjectConfiguration (name);
+			conf.CopyFrom (base.CreateConfiguration (name));
+			conf.OutputDirectory = BaseDirectory.IsNullOrEmpty? "bin" : (string)BaseDirectory.Combine ("bin");			
 			return conf;
 		}
 		
@@ -194,6 +174,13 @@ namespace MonoDevelop.AspNet
 		
 		#endregion
 		
+		public override void Dispose ()
+		{
+			codebehindTypeNameCache.Dispose ();
+			RegistrationCache.Dispose ();
+			base.Dispose ();
+		}
+		
 		#region build/prebuild/execute
 		
 		
@@ -352,7 +339,7 @@ namespace MonoDevelop.AspNet
 		
 		public ProjectDom ResolveAssemblyDom (string assemblyName)
 		{
-			System.Reflection.AssemblyName parsed = SystemAssemblyService.ParseAssemblyName (assemblyName);
+			var parsed = SystemAssemblyService.ParseAssemblyName (assemblyName);
 			if (string.IsNullOrEmpty (parsed.Name))
 				return null;
 			
@@ -381,7 +368,7 @@ namespace MonoDevelop.AspNet
 		
 		string GetAssemblyPath (string assemblyName)
 		{
-			System.Reflection.AssemblyName parsed = SystemAssemblyService.ParseAssemblyName (assemblyName);
+			var parsed = SystemAssemblyService.ParseAssemblyName (assemblyName);
 			if (string.IsNullOrEmpty (parsed.Name))
 				return null;
 			
@@ -470,8 +457,8 @@ namespace MonoDevelop.AspNet
 		
 		void UpdateWebConfigRefs ()
 		{
-			List<string> refs = new List<string> ();
-			foreach (ProjectReference reference in References) {
+			var refs = new List<string> ();
+			foreach (var reference in References) {
 				//local copied assemblies are copied to the bin directory so ASP.NET references them automatically
 				if (reference.LocalCopy && (reference.ReferenceType == ReferenceType.Project || reference.ReferenceType == ReferenceType.Assembly))
 					continue;
@@ -487,15 +474,14 @@ namespace MonoDevelop.AspNet
 				refs.Add (reference.Reference);
 			}
 						
-			string webConfigPath = WebConfigPath;
-			if (!File.Exists (webConfigPath))
+			var webConfig = GetWebConfig ();
+			if (webConfig == null || !File.Exists (webConfig.FilePath))
 				return;
 			
-			MonoDevelop.Projects.Text.IEditableTextFile textFile = 
-				MonoDevelop.DesignerSupport.OpenDocumentFileProvider.Instance.GetEditableTextFile (webConfigPath);
+			var textFile = DesignerSupport.OpenDocumentFileProvider.Instance.GetEditableTextFile (webConfig.FilePath);
 			//use textfile API because it's write safe (writes out to another file then moves)
 			if (textFile == null)
-				textFile = MonoDevelop.Projects.Text.TextFile.ReadFile (webConfigPath);
+				textFile = MonoDevelop.Projects.Text.TextFile.ReadFile (webConfig.FilePath);
 				
 			//can't use System.Web.Configuration.WebConfigurationManager, as it can only access virtual paths within an app
 			//so need full manual handling
@@ -583,8 +569,19 @@ namespace MonoDevelop.AspNet
 			return result;
 		}
 		
-		string WebConfigPath {
-			get { return Path.Combine (this.BaseDirectory, "web.config"); }
+		ProjectFile GetWebConfig ()
+		{
+			var webConf = this.BaseDirectory.Combine ("web.config");
+			foreach (var file in this.Files)
+				if (string.Compare (file.FilePath.ToString (), webConf, StringComparison.OrdinalIgnoreCase) == 0)
+					return file;
+			return null;
+		}
+		
+		bool IsWebConfig (FilePath file)
+		{
+			var webConf = this.BaseDirectory.Combine ("web.config");
+			return (string.Compare (file, webConf, StringComparison.OrdinalIgnoreCase) == 0);
 		}
 		
 		bool IsSystemReference (string reference)
@@ -631,7 +628,7 @@ namespace MonoDevelop.AspNet
 			IEnumerable<string> filesToAdd = MonoDevelop.DesignerSupport.CodeBehind.GuessDependencies
 				(this, e.ProjectFile, groupedExtensions);
 			
-			if (Path.GetFullPath (e.ProjectFile.FilePath) == Path.GetFullPath (WebConfigPath))
+			if (IsWebConfig (e.ProjectFile.FilePath))
 				UpdateWebConfigRefs ();
 			
 			//let the base fire the event before we add files
@@ -697,27 +694,28 @@ namespace MonoDevelop.AspNet
 			}
 			return null;
 		}
-
+		
 		public string GetCodebehindTypeName (string fileName)
 		{
 			lock (codebehindTypeNameCache)
 				return codebehindTypeNameCache.GetCodeBehindTypeName (fileName);
 		}
 		
-		class CodeBehindTypeNameCache : FileInfoCache<string> 
+		class CodeBehindTypeNameCache : ProjectFileCache<AspNetAppProject,string>
 		{
-			AspNetAppProject proj;
-			
-			public CodeBehindTypeNameCache (AspNetAppProject proj)
+			public CodeBehindTypeNameCache (AspNetAppProject proj) : base (proj)
 			{
-				this.proj = proj;
 			}
 			
-			protected override string GenerateInfo (string fileName)
+			protected override string GenerateInfo (string filename)
 			{
-				var cu = ProjectDomService.Parse (proj, fileName, null) as AspNetParsedDocument;
-				if (cu != null && !string.IsNullOrEmpty (cu.PageInfo.InheritedClass))
-					return cu.PageInfo.InheritedClass;
+				try {
+					var doc = ProjectDomService.Parse (Project, filename, null) as AspNetParsedDocument;
+					if (doc != null && !string.IsNullOrEmpty (doc.Info.InheritedClass))
+						return doc.Info.InheritedClass;
+				} catch (Exception ex) {
+					LoggingService.LogError ("Error reading codebehind name for file '" + filename + "'", ex);
+				}
 				return null;
 			}
 			
