@@ -33,10 +33,8 @@ using System;
 using System.IO;
 using System.Collections.Generic;
 
-using Mono.Addins;
-
 using MonoDevelop.Projects;
-using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide;
 
 namespace CBinding
 {
