@@ -3,6 +3,7 @@
 //
 // Authors:
 //   Marcos David Marin Amador <MarcosMarin@gmail.com>
+//   Mitchell Wheeler <mitchell.wheeler@gmail.com>
 //
 // Copyright (C) 2007 Marcos David Marin Amador
 //
@@ -32,6 +33,7 @@
 using System;
 using System.IO;
 using System.Text;
+using System.Text.RegularExpressions;
 using System.Collections.Generic;
 using System.Threading;
 
@@ -42,6 +44,10 @@ using MonoDevelop.Ide.Gui;
 
 using CBinding.Navigation;
 
+//  TODO
+//  Generic, language independant 'TagDatabaseManager'
+//  Parsing of ctags data into a Sqlite database, for easy/efficient access & updates.
+//  
 namespace CBinding.Parser
 {
 	/// <summary>
@@ -76,7 +82,7 @@ namespace CBinding.Parser
 			get {
 				if (!checkedCtagsInstalled) {
 					checkedCtagsInstalled = true;
-					if (IdeApp.Services.PlatformService.Name.Equals ("OSX")) {
+					if (PropertyService.IsMac) {
 						return false;
 					}
 					try {
@@ -105,75 +111,155 @@ namespace CBinding.Parser
 					ctagsInstalled = false;
 			}
 		}
-
-		private string[] Headers (string filename, bool with_system)
+		
+		private string[] Headers (Project project, string filename, bool with_system)
 		{
-			string option = (with_system ? "-M" : "-MM");
-			ProcessWrapper p;
+			List<string> headers = new List<string> ();
+			CProject cproject = project as CProject;
+			if (cproject == null){ return headers.ToArray(); }
+			
+			StringBuilder output = new StringBuilder ();
+			StringBuilder option = new StringBuilder ("-M");
+			if (!with_system) {
+				option.Append("M");
+			}
+			
+			option.Append (" -MG ");
+			foreach (Package package in cproject.Packages) {
+				package.ParsePackage ();
+				option.AppendFormat ("{0} ", string.Join(" ", package.CFlags.ToArray ()));
+			}
+			
+			ProcessWrapper p = null;
 			try {
-				p = Runtime.ProcessService.StartProcess ("gcc", option + " -MG " + filename, null, null);
-				p.WaitForExit ();
+				p = Runtime.ProcessService.StartProcess ("gcc", option.ToString () + filename.Replace(@"\ ", " ").Replace(" ", @"\ "), null, null);
+				p.WaitForOutput ();
+
+				// Doing the below completely breaks header parsing
+				// // Skip first two lines (.o & .c* files) - WARNING, sometimes this is compacted to 1 line... we need a better way of handling this.
+				// if(p.StandardOutput.ReadLine () == null) return new string[0]; // object file
+				// if(p.StandardOutput.ReadLine () == null) return new string[0]; // compile file
+
+				string line;
+				while ((line = p.StandardOutput.ReadLine ()) != null)
+					output.Append (line);
 			} catch (Exception ex) {
 				LoggingService.LogError (ex.ToString ());
 				return new string [0];
 			}
+			finally {
+				if(p != null)
+					p.Dispose();
+			}
 			
-			StringBuilder output = new StringBuilder ();
-			string line;
-			
-			while ((line = p.StandardOutput.ReadLine ()) != null)
-				output.Append (line);
-			
-			p.Close ();
-			
-			string[] lines = output.ToString ().Split ('\\');
-			List<string> headers = new List<string> ();
-			
-			for (int i = 0; i < lines.Length; i++) {
-				string[] files = lines[i].Split (' ');
-				// first line contains the rule (eg. file.o: dep1.c dep2.h ...) and we must skip it
-				// and we skip the *.cpp or *.c etc. too
-				for (int j = 0; j < files.Length; j++) {
-					if (j == 0 || j == 1) continue;
-					
-					string depfile = files[j].Trim ();
-					
-					if (!string.IsNullOrEmpty (depfile))
-						headers.Add (depfile);
-				}
+			MatchCollection files = Regex.Matches(output.ToString().Replace(@" \", String.Empty), @" (?<file>([^ \\]|(\\ ))+)", RegexOptions.IgnoreCase);
+
+			foreach (Match match in files) {
+				string depfile = findFileInIncludes(project, match.Groups["file"].Value.Trim());
+				
+				headers.Add (depfile.Replace(@"\ ", " ").Replace(" ", @"\ "));
 			}
 			
 			return headers.ToArray ();
 		}
 		
+		/// <summary>
+		/// Finds a file in a project's include path(s)
+		/// </summary>
+		/// <param name="project">
+		/// The project whose include path is to be searched
+		/// <see cref="Project"/>
+		/// </param>
+		/// <param name="filename">
+		/// A portion of a full file path
+		/// <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// The full found path, or filename if not found
+		/// <see cref="System.String"/>
+		/// </returns>
+		private static string findFileInIncludes (Project project, string filename) {
+			CProjectConfiguration conf = project.DefaultConfiguration as CProjectConfiguration;
+			string fullpath = string.Empty;
+			
+			if (!Path.IsPathRooted (filename)) {
+				// Check against base directory
+				fullpath = findFileInPath (filename, project.BaseDirectory);
+				if (string.Empty != fullpath) return fullpath;
+
+				// Check project's additional configuration includes
+				foreach (string p in conf.Includes) {
+					fullpath = findFileInPath (filename, p);
+					if (string.Empty != fullpath) return fullpath;
+				}
+			}
+			
+			return filename;
+		}
+		
+		/// <summary>
+		/// Finds a file in a subdirectory of a given path
+		/// </summary>
+		/// <param name="relativeFilename">
+		/// A portion of a full file path
+		/// <see cref="System.String"/>
+		/// </param>
+		/// <param name="path">
+		/// The path beneath which to look for relativeFilename
+		/// <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// The full path, or string.Empty if not found
+		/// <see cref="System.String"/>
+		/// </returns>
+		private static string findFileInPath (string relativeFilename, string path) {
+			string tmp = Path.Combine (path, relativeFilename);
+			
+			if (Path.IsPathRooted (relativeFilename))
+				return relativeFilename;
+			else if (File.Exists (tmp))
+				return tmp;
+			
+			if (Directory.Exists (path)) {
+				foreach (string subdir in Directory.GetDirectories (path)) {
+					tmp = findFileInPath (relativeFilename, subdir);
+					if (string.Empty != tmp) return tmp;
+				}
+			}
+			
+			return string.Empty;
+		}
+		
 		private void UpdateSystemTags (Project project, string filename, string[] includedFiles)
 		{
 			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
 			List<FileInformation> files;
 			
-			if (!info.IncludedFiles.ContainsKey (filename)) {
-				files = new List<FileInformation> ();
-				info.IncludedFiles.Add (filename, files);
-			} else {
-				files = info.IncludedFiles[filename];
-			}
-			
-			foreach (string includedFile in includedFiles) {
-				bool contains = false;
+			lock (info) {
+				if (!info.IncludedFiles.ContainsKey (filename)) {
+					files = new List<FileInformation> ();
+					info.IncludedFiles.Add (filename, files);
+				} else {
+					files = info.IncludedFiles[filename];
+				}
 				
-				foreach (FileInformation fi in files) {
-					if (fi.FileName == includedFile) {
-						contains = true;
+				foreach (string includedFile in includedFiles) {
+					bool contains = false;
+					
+					foreach (FileInformation fi in files) {
+						if (fi.FileName == includedFile) {
+							contains = true;
+						}
 					}
+					
+					if (!contains) {
+						FileInformation newFileInfo = new FileInformation (project, includedFile);
+						files.Add (newFileInfo);
+						FillFileInformation (newFileInfo);
+					}
+					
+					contains = false;
 				}
-				
-				if (!contains) {
-					FileInformation newFileInfo = new FileInformation (project, includedFile);
-					files.Add (newFileInfo);
-					FillFileInformation (newFileInfo);
-				}
-				
-				contains = false;
 			}
 		}
 		
@@ -186,7 +272,12 @@ namespace CBinding.Parser
 			string tagFileName = Path.GetFileName (fileInfo.FileName) + ".tag";
 			string tagdir = Path.Combine (confdir, "system-tags");
 			string tagFullFileName = Path.Combine (tagdir, tagFileName);
-			string ctags_options = "--C++-kinds=+p+u --fields=+a-f+S --language-force=C++ --excmd=pattern -f '" + tagFullFileName + "' " + fileInfo.FileName;
+			string ctags_kinds = "--C++-kinds=+px";
+			
+			if (PropertyService.Get<bool> ("CBinding.ParseLocalVariables", true))
+				ctags_kinds += "l";
+			
+			string ctags_options = ctags_kinds + " --fields=+aStisk-fz --language-force=C++ --excmd=number --line-directives=yes -f '" + tagFullFileName + "' '" + fileInfo.FileName + "'";
 			
 			if (!Directory.Exists (tagdir))
 				Directory.CreateDirectory (tagdir);
@@ -197,10 +288,10 @@ namespace CBinding.Parser
 				try {
 					output = new System.IO.StringWriter ();
 					
-					p = Runtime.ProcessService.StartProcess ("ctags", ctags_options, null, output, output, null);
+					p = Runtime.ProcessService.StartProcess ("ctags", ctags_options, null, null, output, null);
 					p.WaitForOutput (10000);
 					if (p.ExitCode != 0 || !File.Exists (tagFullFileName)) {
-						LoggingService.LogError ("Ctags did not successfully populate the tags database '{0}' from '{1}' within ten seconds.\nOutput: {2}", tagFullFileName, output.ToString ());
+						LoggingService.LogError ("Ctags did not successfully populate the tags database '{0}' within ten seconds.\nOutput: {1}", tagFullFileName, output.ToString ());
 						return;
 					}
 				} catch (Exception ex) {
@@ -251,6 +342,10 @@ namespace CBinding.Parser
 				DepsInstalled = false;
 				return;
 			}
+//			catch {
+//				LoggingService.LogError("Unexpected error while updating parser database. Disabling C/C++ parsing.");
+//				DepsInstalled = false;
+//			}
 		}
 		
 		public void UpdateFileTags (Project project, string filename)
@@ -267,7 +362,9 @@ namespace CBinding.Parser
 			
 			if (parsingThread == null || !parsingThread.IsAlive) {
 				parsingThread = new Thread (ParsingThread);
+				parsingThread.Name = "Tag database parser";
 				parsingThread.IsBackground = true;
+				parsingThread.Priority = ThreadPriority.Lowest;
 				parsingThread.Start();
 			}
 		}
@@ -276,19 +373,29 @@ namespace CBinding.Parser
 		{
 			if (!DepsInstalled)
 				return;
+
+			string[] headers = Headers (project, filename, false);
+			string[] system_headers = diff (Headers (project, filename, true), headers);
+			StringBuilder ctags_kinds = new StringBuilder ("--C++-kinds=+px");
 			
-			string[] headers = Headers (filename, false);
-			string ctags_options = "--C++-kinds=+p+u --fields=+a-f+S --language-force=C++ --excmd=pattern -f - " + filename + " " + string.Join (" ", headers);
+			if (PropertyService.Get<bool> ("CBinding.ParseLocalVariables", true))
+				ctags_kinds.Append ("+l");
 			
-			string[] system_headers = diff (Headers (filename, true), headers);
+			// Maybe we should only ask for locals for 'local' files? (not external #includes?)
+			ctags_kinds.AppendFormat (" --fields=+aStisk-fz --language-force=C++ --excmd=number --line-directives=yes -f - '{0}'", filename);
+			foreach (string header in headers) {
+				ctags_kinds.AppendFormat (" '{0}'", header);
+			}
 			
+			string ctags_output = string.Empty;
+
 			ProcessWrapper p = null;
 			System.IO.StringWriter output = null, error = null;
-			string ctags_output = string.Empty;
 			try {
 				output = new System.IO.StringWriter ();
 				error = new System.IO.StringWriter ();
-				p = Runtime.ProcessService.StartProcess ("ctags", ctags_options, null, output, error, null);
+				
+				p = Runtime.ProcessService.StartProcess ("ctags", ctags_kinds.ToString (), project.BaseDirectory, output, error, null);
 				p.WaitForOutput (10000);
 				if (p.ExitCode != 0) {
 					LoggingService.LogError ("Ctags did not successfully populate the tags database from '{0}' within ten seconds.\nError output: {1}", filename, error.ToString ());
@@ -307,21 +414,24 @@ namespace CBinding.Parser
 			}
 			
 			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
-			string tagEntry;
-
-			using (StringReader reader = new StringReader (ctags_output)) {
-				while ((tagEntry = reader.ReadLine ()) != null) {
-					if (tagEntry.StartsWith ("!_")) continue;
-					
-					Tag tag = ParseTag (tagEntry);
-					
-					if (tag != null)
-						AddInfo (info, tag, ctags_output);
-				}
-			}			
 			
-			if (FileUpdated != null)
-				FileUpdated (new ClassPadEventArgs (project));
+			lock (info) {
+				info.RemoveFileInfo (filename);
+				string tagEntry;
+	
+				using (StringReader reader = new StringReader (ctags_output)) {
+					while ((tagEntry = reader.ReadLine ()) != null) {
+						if (tagEntry.StartsWith ("!_")) continue;
+						
+						Tag tag = ParseTag (tagEntry);
+						
+						if (tag != null)
+							AddInfo (info, tag, ctags_output);
+					}
+				}
+			}
+			
+			OnFileUpdated (new ClassPadEventArgs (project));
 			
 			if (PropertyService.Get<bool> ("CBinding.ParseSystemTags", true))
 				UpdateSystemTags (project, filename, system_headers);
@@ -357,6 +467,9 @@ namespace CBinding.Parser
 					info.Functions.Add (f);
 				break;
 			case TagKind.Local:
+				Local lo = new Local (tag, info.Project, ctags_output);
+				if(!info.Locals.Contains (lo))
+					info.Locals.Add (lo);
 				break;
 			case TagKind.Macro:
 				Macro m = new Macro (tag, info.Project);
@@ -405,9 +518,8 @@ namespace CBinding.Parser
 		
 		private Tag ParseTag (string tagEntry)
 		{
-			int i1, i2;
 			string file;
-			string pattern;
+			UInt64 line;
 			string name;
 			string tagField;
 			TagKind kind;
@@ -418,30 +530,23 @@ namespace CBinding.Parser
 			string _union = null;
 			string _enum = null;
 			string signature = null;
-			char delimiter;
 			
+			int i1 = tagEntry.IndexOf ('\t');
 			name = tagEntry.Substring (0, tagEntry.IndexOf ('\t'));
 			
-			i1 = tagEntry.IndexOf ('\t') + 1;
-			i2 = tagEntry.IndexOf ('\t', i1);
-			
+			i1 += 1;
+			int i2 = tagEntry.IndexOf ('\t', i1);
 			file = tagEntry.Substring (i1, i2 - i1);
 			
-			delimiter = tagEntry[i2 + 1];
+			i1 = i2 + 1;
+			i2 = tagEntry.IndexOf (";\"", i1);
+			line = UInt64.Parse(tagEntry.Substring (i1, i2 - i1));
+
+			i1 = i2 + 3;	
+			kind = (TagKind)tagEntry[i1];
 			
-			i1 = i2 + 2;
-			i2 = tagEntry.IndexOf (delimiter, i1) - 1;
-			
-			// apparentlty sometimes ctags will create faulty tags, make sure this is not one of them
-			if (i2 < 0 || i1 < 0)
-				return null;
-			
-			pattern = tagEntry.Substring (i1 + 1, i2 - i1 - 1);
-			
-			tagField = tagEntry.Substring (i2 + 5);
-			
-			// parse tag field
-			kind = (TagKind)tagField[0];
+			i1 += 2;
+			tagField = (tagEntry.Length > i1? tagField = tagEntry.Substring(i1) : String.Empty);
 			
 			string[] fields = tagField.Split ('\t');
 			int index;
@@ -455,35 +560,35 @@ namespace CBinding.Parser
 					string val = field.Substring (index + 1);
 					
 					switch (key) {
-					case "access":
-						try {
-							access = (AccessModifier)System.Enum.Parse (typeof(AccessModifier), val, true);
-						} catch (ArgumentException) {
-						}
-						break;
-					case "class":
-						_class = val;
-						break;
-					case "namespace":
-						_namespace = val;
-						break;
-					case "struct":
-						_struct = val;
-						break;
-					case "union":
-						_union = val;
-						break;
-					case "enum":
-						_enum = val;
-						break;
-					case "signature":
-						signature = val;
-						break;
+						case "access":
+							try {
+								access = (AccessModifier)System.Enum.Parse (typeof(AccessModifier), val, true);
+							} catch (ArgumentException) {
+							}
+							break;
+						case "class":
+							_class = val;
+							break;
+						case "namespace":
+							_namespace = val;
+							break;
+						case "struct":
+							_struct = val;
+							break;
+						case "union":
+							_union = val;
+							break;
+						case "enum":
+							_enum = val;
+							break;
+						case "signature":
+							signature = val;
+							break;
 					}
 				}
 			}
 			
-			return new Tag (name, file, pattern, kind, access, _class, _namespace, _struct, _union, _enum, signature);
+			return new Tag (name, file, line, kind, access, _class, _namespace, _struct, _union, _enum, signature);
 		}
 		
 		Tag BinarySearch (string[] ctags_lines, TagKind kind, string name)
@@ -598,6 +703,22 @@ namespace CBinding.Parser
 			}
 		}
 		
+		/// <summary>
+		/// Remove a file's parse information from the database.
+		/// </summary>
+		/// <param name="project">
+		/// A <see cref="Project"/>: The project to which the file belongs.
+		/// </param>
+		/// <param name="filename">
+		/// A <see cref="System.String"/>: The file.
+		/// </param>
+		public void RemoveFileInfo(Project project, string filename)
+		{
+			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
+			lock (info){ info.RemoveFileInfo(filename); }
+			OnFileUpdated(new ClassPadEventArgs (project));
+		}
+		
 		private static string[] diff (string[] a1, string[] a2)
 		{
 			List<string> res = new List<string> ();
@@ -611,6 +732,14 @@ namespace CBinding.Parser
 			return res.ToArray ();
 		}
 		
+		/// <summary>
+		/// Wrapper method for the FileUpdated event.
+		/// </summary>
+		void OnFileUpdated (ClassPadEventArgs args)
+		{
+			if (null != FileUpdated){ FileUpdated(args); }
+		}
+		
 		private class ProjectFilePair
 		{
 			string file;
