@@ -32,24 +32,20 @@
 using System;
 using System.IO;
 using System.Xml;
-using System.Collections;
 using System.Collections.Generic;
 using System.ComponentModel;
-using System.CodeDom.Compiler;
 
 using Mono.Addins;
 
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Projects;
 using MonoDevelop.Core.Serialization;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Deployment;
 using MonoDevelop.Deployment.Linux;
-
 using CBinding.Parser;
+using MonoDevelop.Ide;
 
 namespace CBinding
 {
@@ -61,7 +57,6 @@ namespace CBinding
 	public enum CProjectCommands {
 		AddPackage,
 		UpdateClassPad,
-		SwapSourceHeader,
 		ShowPackageDetails
 	}
 	
@@ -154,6 +149,10 @@ namespace CBinding
 						    typeof(CBinding.CompileTarget),
 						    projectOptions.Attributes["Target"].InnerText);
 					}
+					if (projectOptions.GetAttribute ("ExternalConsole") == "True") {
+						c.ExternalConsole = true;
+						c.PauseConsoleOutput = true;
+					}
 					if (projectOptions.Attributes["PauseConsoleOutput"] != null) {
 						c.PauseConsoleOutput = bool.Parse (
 							projectOptions.Attributes["PauseConsoleOutput"].InnerText);
@@ -239,7 +238,9 @@ namespace CBinding
 				writer.WriteLine ("Name: {0}", Name);
 				writer.WriteLine ("Description: {0}", Description);
 				writer.WriteLine ("Version: {0}", Version);
-				writer.WriteLine ("Libs: -L{0} -l{1}", config.OutputDirectory, config.Output);
+				writer.WriteLine ("Libs: -L{0} -l{1}", config.OutputDirectory, config.Output.StartsWith ("lib", StringComparison.OrdinalIgnoreCase)?
+				                                                                                                config.Output.Substring (3):
+				                                                                                                config.Output);
 //				writer.WriteLine ("Cflags: -I{0}", BaseDirectory);
 				writer.WriteLine ("Cflags: -I{0}", string.Join (" -I", headerDirectories.ToArray ()));
 			}
@@ -280,7 +281,9 @@ namespace CBinding
 				// TODO: How should I get this?
 				writer.WriteLine ("Conflicts: {0}", string.Empty);
 				writer.Write ("Libs: -L${libdir} ");
-				writer.WriteLine ("-l{0}", config.Output);
+				writer.WriteLine ("-l{0}", config.Output.StartsWith ("lib", StringComparison.OrdinalIgnoreCase)?
+				                                                            config.Output.Substring (3):
+				                                                            config.Output);
 				writer.Write ("Cflags: -I${includedir}/");
 				writer.WriteLine ("{0} {1}", Name, Compiler.GetDefineFlags (project, config));
 			}
@@ -473,7 +476,7 @@ namespace CBinding
 					string targetDirectory =
 						(IsHeaderFile (f.Name) ? TargetDirectory.Include : TargetDirectory.ProgramFiles);
 					
-					deployFiles.Add (new DeployFile (this, f.FilePath, f.RelativePath, targetDirectory));
+					deployFiles.Add (new DeployFile (this, f.FilePath, f.ProjectVirtualPath, targetDirectory));
 				}
 			}
 			
