@@ -58,25 +58,50 @@ namespace MonoDevelop.CSharp.Refactoring
 		}
 		
 		NRefactoryResolver resolver;
-		IEditableTextFile file;
 		
 		MonoDevelop.Projects.Dom.INode searchedMember;
 		DomLocation   searchedMemberLocation;
 		string        searchedMemberName;
 		string        searchedMemberFile = null;
 		
+		string fileName;
+		Mono.TextEditor.Document text;
+		
 		Stack<TypeDeclaration> typeStack = new Stack<TypeDeclaration> ();
 		
+		public string SearchedMemberName {
+			get { return this.searchedMemberName; }
+		}
+		
 		public bool IncludeXmlDocumentation {
 			get;
 			set;
 		}
+		
+		public FindMemberAstVisitor (Mono.TextEditor.Document document, NRefactoryResolver resolver, MonoDevelop.Projects.Dom.INode searchedMember)
+		{
+			fileName = document.FileName;
+			this.text = document;
+			Init (resolver, searchedMember);
+		}
+		
 		public FindMemberAstVisitor (NRefactoryResolver resolver, IEditableTextFile file, MonoDevelop.Projects.Dom.INode searchedMember)
 		{
-			this.file = file;
+			fileName = file.Name;
+			text = new Mono.TextEditor.Document ();
+			text.Text = file.Text;
+			Init (resolver, searchedMember);
+		}
+		
+		void Init (NRefactoryResolver resolver, MonoDevelop.Projects.Dom.INode searchedMember)
+		{
 			this.resolver = resolver;
+			if (searchedMember is IMember) 
+				searchedMember = GetUnderlyingMember ((IMember)searchedMember);
+			
 			this.searchedMember = searchedMember;
 			this.IncludeXmlDocumentation = false;
+			
 			if (searchedMember is IMethod) {
 				IMethod method = (IMethod)searchedMember;
 				this.searchedMemberName = method.IsConstructor ? method.DeclaringType.Name : method.Name;
@@ -106,17 +131,48 @@ namespace MonoDevelop.CSharp.Refactoring
 			}
 		}
 		
+		// if a member is a member of an instantiated class search for the 'real' uninstantiated member
+		// ex. List<string> a; a.Count<-;  should search for List<T>.Count instead of List<string>.Count
+		static IMember GetUnderlyingMember (IMember member)
+		{
+			if (member == null)
+				return null;
+			
+			if (member.DeclaringType is InstantiatedType && member.ReturnType != null) {
+				IType uninstantiatedType = ((InstantiatedType)member.DeclaringType).UninstantiatedType;
+				foreach (IMember realMember in uninstantiatedType.SearchMember (member.Name, true)) {
+					if (realMember.ReturnType == null)
+						continue;
+					if (realMember.MemberType == member.MemberType) {
+						switch (member.MemberType) {
+						case MemberType.Method:
+							if (((IMethod)member).TypeParameters.Count !=  ((IMethod)realMember).TypeParameters.Count)
+								continue;
+							if (!DomMethod.ParameterListEquals (((IMethod)member).Parameters, ((IMethod)realMember).Parameters))
+								continue;
+							break;
+						case MemberType.Property:
+							if (!DomMethod.ParameterListEquals (((IProperty)member).Parameters, ((IProperty)realMember).Parameters))
+								continue;
+							break;
+						}
+						return realMember;
+					}
+				}
+			}
+			return member;
+		}
+		
 		static readonly Regex paramRegex    = new Regex ("\\<param\\s+name\\s*=\\s*\"(.*)\"", RegexOptions.Compiled);
 		static readonly Regex paramRefRegex = new Regex ("\\<paramref\\s+name\\s*=\\s*\"(.*)\"", RegexOptions.Compiled);
 
 		static readonly Regex seeRegex       = new Regex ("\\<see\\s+cref\\s*=\\s*\"(.*)\"", RegexOptions.Compiled);
 		static readonly Regex seeAlsoRegRegex = new Regex ("\\<seealso\\s+cref\\s*=\\s*\"(.*)\"", RegexOptions.Compiled);
-
+		
 		public void RunVisitor ()
 		{
 			if (searchedMember == null)
 				return;
-			string text = file.Text;
 			
 			// search if the member name exists in the file (otherwise it doesn't make sense to search it)
 			FindReplace findReplace = new FindReplace ();
@@ -125,13 +181,13 @@ namespace MonoDevelop.CSharp.Refactoring
 				WholeWordsOnly = true
 			};
 			findReplace.CompilePattern (searchedMemberName, filterOptions);
-			IEnumerable<SearchResult> result = findReplace.Search (new FileProvider (null), text, searchedMemberName, null, filterOptions);
+			IEnumerable<SearchResult> result = findReplace.Search (new FileProvider (null), text.Text, searchedMemberName, null, filterOptions);
 			if (result == null || !result.Any ()) {
 				return;
 			}
 			
-			// It exists -> now looking with the parser
-			ICSharpCode.NRefactory.IParser parser = ICSharpCode.NRefactory.ParserFactory.CreateParser (ICSharpCode.NRefactory.SupportedLanguage.CSharp, new StringReader (text));
+			string parseText = text.Text;
+			ICSharpCode.NRefactory.IParser parser = ICSharpCode.NRefactory.ParserFactory.CreateParser (ICSharpCode.NRefactory.SupportedLanguage.CSharp, new StringReader (parseText));
 			parser.Lexer.EvaluateConditionalCompilation = true;
 			parser.Parse ();
 			resolver.SetupParsedCompilationUnit (parser.CompilationUnit);
@@ -142,11 +198,13 @@ namespace MonoDevelop.CSharp.Refactoring
 				parser.Lexer.ConditionalCompilationSymbols.Clear ();
 				foreach (string define in usedIdentifiers[i])
 					parser.Lexer.ConditionalCompilationSymbols.Add (define, true);
-				parser = ICSharpCode.NRefactory.ParserFactory.CreateParser (ICSharpCode.NRefactory.SupportedLanguage.CSharp, new StringReader (file.Text));
+				parser.Dispose ();
+				parser = ICSharpCode.NRefactory.ParserFactory.CreateParser (ICSharpCode.NRefactory.SupportedLanguage.CSharp, new StringReader (parseText));
 				parser.Parse ();
 				
 				VisitCompilationUnit (parser.CompilationUnit, null);
 			}
+			
 			if (IncludeXmlDocumentation) {
 				if (searchedMember is IParameter) {
 					IParameter parameter = (IParameter)searchedMember;
@@ -191,8 +249,30 @@ namespace MonoDevelop.CSharp.Refactoring
 					}
 				}
 			}
+			parser.Dispose ();
 		}
-
+		
+		
+		public void RunVisitor (ICSharpCode.NRefactory.Ast.CompilationUnit compilationUnit)
+		{
+			if (searchedMember == null)
+				return;
+			// search if the member name exists in the file (otherwise it doesn't make sense to search it)
+			FindReplace findReplace = new FindReplace ();
+			FilterOptions filterOptions = new FilterOptions {
+				CaseSensitive = true,
+				WholeWordsOnly = true
+			};
+			findReplace.CompilePattern (searchedMemberName, filterOptions);
+			IEnumerable<SearchResult> result = findReplace.Search (new FileProvider (null), text.Text, searchedMemberName, null, filterOptions);
+			if (result == null || !result.Any ()) {
+				return;
+			}
+			
+			resolver.SetupParsedCompilationUnit (compilationUnit);
+			VisitCompilationUnit (compilationUnit, null);
+		}
+		
 		class ExpressionVisitor : ICSharpCode.NRefactory.Visitors.AbstractAstVisitor
 		{
 			HashSet<string> identifiers = new HashSet<string> ();
@@ -263,10 +343,10 @@ namespace MonoDevelop.CSharp.Refactoring
 		
 		MemberReference CreateReference (int line, int col, string name)
 		{
-			int pos = file.GetPositionFromLineColumn (line, col);
-			int spos = file.GetPositionFromLineColumn (line, 1);
-			int epos = file.GetPositionFromLineColumn (line + 1, 1);
-			if (epos == -1) epos = file.Length - 1;
+			int pos = text.LocationToOffset (line - 1, col - 1);
+			int spos = text.LocationToOffset (line - 1, 0);
+			int epos = text.LocationToOffset (line, 0);
+			if (epos == -1) epos = text.Length - 1;
 			
 			string txt;
 			
@@ -278,16 +358,16 @@ namespace MonoDevelop.CSharp.Refactoring
 			// update @pos to point to the actual identifier and not the 
 			// public/private/whatever modifier.
 			int i;
-			txt = file.GetText (pos, file.Length - 1);
+			txt = text.GetTextBetween (pos, text.Length - 1);
 			if (txt != null && (i = txt.IndexOf (name)) > 0)
 				pos += i;
 			
 			if (spos != -1)
-				txt = file.GetText (spos, epos - 1);
+				txt = text.GetTextBetween (spos, epos - 1);
 			else
 				txt = null;
 			
-			return new MemberReference (null, file.Name, pos, line, col, name, txt);
+			return new MemberReference (null, fileName, pos, line, col, name, txt);
 		}
 		
 		HashSet<MemberReference> unique = new HashSet<MemberReference> ();
@@ -311,14 +391,14 @@ namespace MonoDevelop.CSharp.Refactoring
 			
 			if (searchedMember is CompoundType) {
 				foreach (IType part in ((CompoundType)searchedMember).Parts) {
-					if (file.Name == part.CompilationUnit.FileName &&
+					if (fileName == part.CompilationUnit.FileName &&
 					    node.StartLocation.Line == part.Location.Line && 
 					    node.StartLocation.Column == part.Location.Column)
 						return true;
 				}
 			}
 			
-			return (string.IsNullOrEmpty (searchedMemberFile) || file.Name == searchedMemberFile) && 
+			return (string.IsNullOrEmpty (searchedMemberFile) || fileName == searchedMemberFile) && 
 			       node.StartLocation.Line == this.searchedMemberLocation.Line && 
 			       node.StartLocation.Column == this.searchedMemberLocation.Column;
 		}
@@ -330,13 +410,20 @@ namespace MonoDevelop.CSharp.Refactoring
 		
 		bool SearchText (string text, int startLine, int startColumn, out int line, out int column)
 		{
-			int position = file.GetPositionFromLineColumn (startLine, startColumn);
+			int position = this.text.LocationToOffset (startLine - 1, startColumn - 1);
 			line = column = -1;
-			while (position + searchedMemberName.Length < file.Length) {
-				if ((position == 0 || !IsIdentifierPart (file.GetCharAt (position - 1))) && 
-				    (position + searchedMemberName.Length >= file.Length  || !IsIdentifierPart (file.GetCharAt (position + searchedMemberName.Length))) &&
-				    file.GetText (position, position + searchedMemberName.Length) == searchedMemberName) {
-					file.GetLineColumnFromPosition (position, out line, out column);
+			if (position < 0)
+				return false;
+			
+			while (position + searchedMemberName.Length < this.text.Length) {
+				bool isIdentifierStart = position <= 0 || !IsIdentifierPart (this.text.GetCharAt (position - 1));
+				
+				if (isIdentifierStart &&
+				    (position + searchedMemberName.Length >= this.text.Length  || !IsIdentifierPart (this.text.GetCharAt (position + searchedMemberName.Length))) &&
+				    (this.text.GetTextAt (position, searchedMemberName.Length) == searchedMemberName)) { 
+					var location = this.text.OffsetToLocation (position);
+					line = location.Line + 1;
+					column = location.Column + 1;
 					return true;
 				}
 				position ++;
@@ -344,6 +431,18 @@ namespace MonoDevelop.CSharp.Refactoring
 			return false;
 		}
 		
+		bool IsSearchTextAt (int startLine, int startColumn)
+		{
+			int position = this.text.LocationToOffset (startLine - 1, startColumn - 1);
+			
+			if ((position == 0 || !IsIdentifierPart (this.text.GetCharAt (position - 1))) && 
+			    (position + searchedMemberName.Length >= this.text.Length  || !IsIdentifierPart (this.text.GetCharAt (position + searchedMemberName.Length))) &&
+			    this.text.GetTextAt (position, searchedMemberName.Length) == searchedMemberName) {
+				return true;
+			}
+			return false;
+		}
+		
 		void CheckNode (ICSharpCode.NRefactory.Ast.INode node)
 		{
 			if (IsSearchedNode (node)) {
@@ -366,15 +465,17 @@ namespace MonoDevelop.CSharp.Refactoring
 		{
 			CheckNode (constructorDeclaration);
 			if (this.searchedMember is IType) {
-				if (((IType)this.searchedMember).FullName == CurrentTypeFullName &&
-				    ((IType)this.searchedMember).TypeParameters.Count == typeStack.Peek ().Templates.Count)
+				if (((IType)this.searchedMember).Parts.Any (t => t.CompilationUnit.FileName == fileName) &&
+				    ((IType)this.searchedMember).FullName == CurrentTypeFullName &&
+				    ((IType)this.searchedMember).TypeParameters.Count == typeStack.Peek ().Templates.Count &&
+				    IsSearchTextAt (constructorDeclaration.StartLocation.Line, constructorDeclaration.StartLocation.Column))
 					AddUniqueReference (constructorDeclaration.StartLocation.Line, constructorDeclaration.StartLocation.Column, this.searchedMemberName);
 			}
 			if (this.searchedMember is IMethod) {
 				IMethod method = (IMethod)this.searchedMember; 
-				if (method.IsConstructor)
+				if (method.IsConstructor &&
+				    IsSearchTextAt (constructorDeclaration.StartLocation.Line, constructorDeclaration.StartLocation.Column))
 					CheckNode (constructorDeclaration);
-				
 			}
 			
 			return base.VisitConstructorDeclaration (constructorDeclaration, data);
@@ -384,8 +485,10 @@ namespace MonoDevelop.CSharp.Refactoring
 		{
 			CheckNode (destructorDeclaration);
 			if (this.searchedMember is IType) {
-				if (((IType)this.searchedMember).FullName == CurrentTypeFullName && 
-				    ((IType)this.searchedMember).TypeParameters.Count == typeStack.Peek ().Templates.Count)
+				if (((IType)this.searchedMember).Parts.Any (t => t.CompilationUnit.FileName == fileName) &&
+				    ((IType)this.searchedMember).FullName == CurrentTypeFullName && 
+				    ((IType)this.searchedMember).TypeParameters.Count == typeStack.Peek ().Templates.Count && 
+				    IsSearchTextAt (destructorDeclaration.StartLocation.Line, destructorDeclaration.StartLocation.Column + 1))
 					AddUniqueReference (destructorDeclaration.StartLocation.Line, destructorDeclaration.StartLocation.Column + 1, this.searchedMemberName);
 			}
 			
@@ -424,10 +527,12 @@ namespace MonoDevelop.CSharp.Refactoring
 				while (parent != null && !(parent is MemberNode) && !(parent is ParametrizedNode)) {
 					parent = parent.Parent;
 				}
+				
 				if (parent != null &&
+					localVariableDeclaration.StartLocation.Line == searchedVariable.Location.Line && 
+					localVariableDeclaration.StartLocation.Column == searchedVariable.Location.Column && 
 				    parent.StartLocation.Line == searchedVariable.DeclaringMember.Location.Line && 
-				    parent.StartLocation.Column == searchedVariable.DeclaringMember.Location.Column &&
-				    searchedVariable.Region.Start != ConvertLocation (localVariableDeclaration.StartLocation)) {
+				    parent.StartLocation.Column == searchedVariable.DeclaringMember.Location.Column) {
 					foreach (VariableDeclaration decl in localVariableDeclaration.Variables) {
 						if (decl.Name == searchedMemberName) 
 							AddUniqueReference (decl.StartLocation.Y, decl.StartLocation.X, searchedMemberName);
@@ -534,9 +639,12 @@ namespace MonoDevelop.CSharp.Refactoring
 				int line = idExp.StartLocation.Y;
 				int col = idExp.StartLocation.X;
 				ResolveResult result = resolver.ResolveIdentifier (idExp.Identifier, ConvertLocation (idExp.StartLocation));
-				//Console.WriteLine ("result:" + result);
+				
 				if (searchedMember is IType) {
-					IMember item = result != null ? ((MemberResolveResult)result).ResolvedMember : null;
+					IMember item = result != null && result is MemberResolveResult ? ((MemberResolveResult)result).ResolvedMember : null;
+					if (item == null && result != null)
+						item = resolver.Dom.GetType (result.ResolvedType);
+					
 					if (item != null && item is IType && ((IType)item).FullName == ((IType)searchedMember).FullName) {
 					//	Debug ("adding IdentifierExpression class", idExp.Identifier, idExp);
 						AddUniqueReference (line, col, idExp.Identifier);
@@ -545,7 +653,7 @@ namespace MonoDevelop.CSharp.Refactoring
 					LocalVariable avar = searchedMember as LocalVariable;
 					LocalVariable var = ((LocalVariableResolveResult)result).LocalVariable;
 
-					if (var != null && avar.DeclaringMember.FullName == var.DeclaringMember.FullName) {
+					if (var != null && avar.DeclaringMember != null && var.DeclaringMember != null && avar.DeclaringMember.FullName == var.DeclaringMember.FullName) {
 //						Console.WriteLine (avar.Region.Start.Line + "---" +  var.Region.Start.Line);
 						if (Math.Abs (avar.Region.Start.Line - var.Region.Start.Line) <= 1)
 							AddUniqueReference (line, col, idExp.Identifier);
@@ -555,12 +663,12 @@ namespace MonoDevelop.CSharp.Refactoring
 					if (param != null && ((IParameter)searchedMember).DeclaringMember.Location == param.DeclaringMember.Location)
 						AddUniqueReference (line, col, idExp.Identifier);
 				} else if (searchedMember is IMember && result is MemberResolveResult) {
-					IMember item = ((MemberResolveResult)result).ResolvedMember;
-					IMember m = item as IMember;
+					IMember m = GetUnderlyingMember (((MemberResolveResult)result).ResolvedMember);
+					
 					//Console.WriteLine (searchedMember +  "/" + item);
-					if (m != null /*&& IsExpectedClass (m.DeclaringType)*/ && ((IMember)searchedMember).DeclaringType.FullName == item.DeclaringType.FullName &&
-						((searchedMember is IField && item is IField) || (searchedMember is IMethod && item is IMethod) ||
-						 (searchedMember is IProperty && item is IProperty) || (searchedMember is IEvent && item is IEvent))) {
+					if (m != null /*&& IsExpectedClass (m.DeclaringType)*/ && ((IMember)searchedMember).DeclaringType.FullName == m.DeclaringType.FullName &&
+						((searchedMember is IField && m is IField) || (searchedMember is IMethod && m is IMethod) ||
+						 (searchedMember is IProperty && m is IProperty) || (searchedMember is IEvent && m is IEvent))) {
 						//Debug ("adding IdentifierExpression searchedMember", searchedMember.Name, idExp);
 						AddUniqueReference (line, col, searchedMemberName);
 					}
@@ -574,15 +682,19 @@ namespace MonoDevelop.CSharp.Refactoring
 		{
 			string type = typeReference.Type;
 			if (searchedMember is IType && this.searchedMemberName == GetNameWithoutPrefix (type)) {
-				int line = typeReference.StartLocation.Y;
-				int col  = typeReference.StartLocation.X;
 				ExpressionResult res = new ExpressionResult ("new " + typeReference.ToString () + "()");
 				ResolveResult resolveResult = resolver.Resolve (res, ConvertLocation (typeReference.StartLocation));
 				
 				IReturnType cls = resolveResult != null ? resolveResult.ResolvedType : null;
-				IType resolvedType = cls != null ? resolver.Dom.SearchType (new SearchTypeRequest (resolver.Unit, cls, resolver.CallingType)) : null;
-				if (resolvedType != null && resolvedType.FullName == ((IType)searchedMember).FullName) 
-					AddUniqueReference (line, col, typeReference.Type);
+				IType resolvedType = cls != null ? resolver.SearchType (cls) : null;
+				if (resolvedType != null && resolvedType.FullName == ((IType)searchedMember).FullName) {
+					int line, column;
+					if (!SearchText (searchedMemberName, typeReference.StartLocation.Line, typeReference.StartLocation.Column, out line, out column)) {
+						line = typeReference.StartLocation.Line;
+						column = typeReference.StartLocation.Column;
+					}
+					AddUniqueReference (line, column, typeReference.Type);
+				}
 			}
 			return base.VisitTypeReference (typeReference, data);
 		}
@@ -592,10 +704,10 @@ namespace MonoDevelop.CSharp.Refactoring
 			if (!(searchedMember is IParameter) && fieldExp.MemberName == searchedMemberName) {
 				ResolveResult resolveResult = resolver.ResolveExpression (fieldExp, ConvertLocation (fieldExp.StartLocation));
 				MemberResolveResult mrr = resolveResult as MemberResolveResult;
-				//Console.WriteLine ("RR:" + resolveResult);
 				if (mrr != null) {
-					//Console.WriteLine ("resolved member:" + mrr.ResolvedMember);
-					if (mrr.ResolvedMember != null && mrr.ResolvedMember.Location == searchedMemberLocation && mrr.ResolvedMember.DeclaringType.FullName == ((IMember)searchedMember).DeclaringType.FullName) {
+					IMember resolvedMember = GetUnderlyingMember (mrr.ResolvedMember);
+					
+					if (resolvedMember != null && resolvedMember.DeclaringType != null && ((IMember)searchedMember).DeclaringType != null && resolvedMember.Location == searchedMemberLocation && resolvedMember.DeclaringType.FullName == ((IMember)searchedMember).DeclaringType.FullName) {
 						int line, column;
 						if (SearchText (searchedMemberName, fieldExp.StartLocation.Line, fieldExp.StartLocation.Column, out line, out column))
 							AddUniqueReference (line, column, searchedMemberName);
@@ -605,11 +717,16 @@ namespace MonoDevelop.CSharp.Refactoring
 				}
 				IType cls = resolveResult != null ? resolver.Dom.GetType (resolveResult.ResolvedType) : null;
 				if (cls != null) {
-					int pos = file.GetPositionFromLineColumn (fieldExp.StartLocation.Y, fieldExp.StartLocation.X);
-					int endpos = file.GetPositionFromLineColumn (fieldExp.EndLocation.Y, fieldExp.EndLocation.X);
-					string txt = file.GetText (pos, endpos);
+					int pos = text.LocationToOffset (fieldExp.StartLocation.Y - 1, fieldExp.StartLocation.X - 1);
+					int endpos = text.LocationToOffset (fieldExp.EndLocation.Y - 1, fieldExp.EndLocation.X - 1);
+					string txt = text.GetTextBetween (pos, endpos);
 					if (txt == searchedMemberName) {
-						AddUniqueReference (fieldExp.StartLocation.Y, fieldExp.StartLocation.X, searchedMemberName);
+						int line, column;
+						if (!SearchText (searchedMemberName, fieldExp.StartLocation.Line, fieldExp.StartLocation.Column, out line, out column)) {
+							line = fieldExp.StartLocation.Line;
+							column = fieldExp.StartLocation.Column;
+						}
+						AddUniqueReference (line, column, searchedMemberName);
 					}
 				}
 			}
@@ -636,7 +753,7 @@ namespace MonoDevelop.CSharp.Refactoring
 						MethodResolveResult mrr = (MethodResolveResult)resolveResult;
 						resolvedMethod = mrr.MostLikelyMethod;
 					} else if (resolveResult is MemberResolveResult) {
-						resolvedMethod = ((MemberResolveResult)resolveResult).ResolvedMember as IMethod;
+						resolvedMethod = GetUnderlyingMember (((MemberResolveResult)resolveResult).ResolvedMember) as IMethod;
 					}
 					if (resolvedMethod != null) {
 						if (resolvedMethod.FullName == method.FullName && resolvedMethod.TypeParameters.Count == method.TypeParameters.Count) {
