@@ -179,9 +179,30 @@ namespace MonoDevelop.CSharp.Resolver
 					
 					ExpressionResult firstExprs = FindExpression (documentToCursor, j);
 					if (firstExprs.Expression != null) {
+						if (documentToCursor[j] == '[') {
+							StringBuilder expr = new StringBuilder (firstExprs.Expression);
+							while (j < documentToCursor.Length) {
+								char ch = documentToCursor[j];
+								switch (ch) {
+								case '[':
+								case ',':
+								case ']':
+									expr.Append (ch);
+									break;
+								case ' ':
+								case '\t':
+									break;
+								default:
+									j = documentToCursor.Length;
+									break;
+								}
+								j++;
+							}
+							firstExprs.Expression = expr.ToString ();
+						}
 						IReturnType unresolvedReturnType = NRefactoryResolver.ParseReturnType (firstExprs);
 						if (unresolvedReturnType != null) {
-							IType resolvedType = projectContent.SearchType (new SearchTypeRequest (unit, unresolvedReturnType, callingType));
+							IType resolvedType = projectContent.SearchType ((INode)callingType ?? unit, unresolvedReturnType);
 							return ExpressionContext.TypeDerivingFrom (resolvedType != null ? new DomReturnType (resolvedType) : null, unresolvedReturnType, true);
 						}
 					}
@@ -230,7 +251,7 @@ namespace MonoDevelop.CSharp.Resolver
 			if (firstExprs.Expression != null) {
 				IReturnType unresolvedReturnType = NRefactoryResolver.ParseReturnType (firstExprs);
 				if (unresolvedReturnType != null) {
-					IType resolvedType = projectContent.SearchType (new SearchTypeRequest (unit, unresolvedReturnType, callingType));
+					IType resolvedType = projectContent.SearchType ((INode)callingType ?? unit, unresolvedReturnType);
 					return ExpressionContext.TypeDerivingFrom (resolvedType != null ? new DomReturnType (resolvedType) : null, unresolvedReturnType, true);
 				}
 				
@@ -308,7 +329,7 @@ namespace MonoDevelop.CSharp.Resolver
 			if (firstExprs.Expression != null) {
 				IReturnType unresolvedReturnType = NRefactoryResolver.ParseReturnType (firstExprs);
 				if (unresolvedReturnType != null) {
-					IType resolvedType = projectContent.SearchType (new SearchTypeRequest (unit, unresolvedReturnType, callingType));
+					IType resolvedType = projectContent.SearchType ((INode)callingType ?? unit, unresolvedReturnType);
 					return ExpressionContext.TypeDerivingFrom (resolvedType != null ? new DomReturnType (resolvedType) : null, unresolvedReturnType, true);
 				}
 				
@@ -687,49 +708,61 @@ namespace MonoDevelop.CSharp.Resolver
 		Frame frame;
 		int lastToken;
 
+/*		public ExpressionResult FindExpression (string text, int offset)
+		{
+			int wordEnd = offset;
+			while (wordEnd < text.Length && (Char.IsLetterOrDigit (text[wordEnd]) || text[wordEnd] == '_'))
+				wordEnd++;
+			return InternalFindExpression (text, wordEnd);
+		}*/
+		
 		public ExpressionResult FindExpression (string text, int offset)
 		{
 			Init (text, offset);
-			Token token;
-			Location lastError = Location.Empty;
-			lexer.Errors.Error = delegate(int errorLine, int errorCol, string errorMsg) { lastError = new Location (errorCol, errorLine); };
-			while ((token = lexer.NextToken ()) != null) {
-				if (token.Kind == Tokens.EOF)
-					break;
-				
-				if (targetPosition <= token.Location) {
-					break;
-				}
-				ApplyToken (token);
-				if (targetPosition <= token.EndLocation) {
-					if (token.Kind == Tokens.Literal) {
-						// do not return string literal as expression if offset was inside the literal,
-						// or if it was at the end of the literal when the literal was not terminated correctly.
-						if (targetPosition < token.EndLocation || lastError == token.Location) {
-							frame.lastExpressionStart = Location.Empty;
-						}
+			try {
+				Token token;
+				Location lastError = Location.Empty;
+				lexer.Errors.Error = delegate(int errorLine, int errorCol, string errorMsg) { lastError = new Location (errorCol, errorLine); };
+				while ((token = lexer.NextToken ()) != null) {
+					if (token.Kind == Tokens.EOF)
+						break;
+					
+					if (targetPosition <= token.Location) {
+						break;
 					}
-					break;
+					ApplyToken (token);
+					if (targetPosition <= token.EndLocation) {
+						if (token.Kind == Tokens.Literal) {
+							// do not return string literal as expression if offset was inside the literal,
+							// or if it was at the end of the literal when the literal was not terminated correctly.
+							if (targetPosition < token.EndLocation || lastError == token.Location) {
+								frame.lastExpressionStart = Location.Empty;
+							}
+						}
+						break;
+					}
+					lastToken = token.Kind;
 				}
-				lastToken = token.Kind;
-			}
-			
-			int tokenOffset;
-			if (token == null || token.Kind == Tokens.EOF)
-				tokenOffset = text.Length;
-			else
-				tokenOffset = LocationToOffset (token.Location);
-			int lastExpressionStartOffset = LocationToOffset (frame.lastExpressionStart);
-			if (lastExpressionStartOffset >= 0) {
-				if (offset < tokenOffset) {
-					// offset is in front of this token
-					return MakeResult (text, lastExpressionStartOffset, tokenOffset, frame.contexts);
+				
+				int tokenOffset;
+				if (token == null || token.Kind == Tokens.EOF)
+					tokenOffset = text.Length;
+				else
+					tokenOffset = LocationToOffset (token.Location);
+				int lastExpressionStartOffset = LocationToOffset (frame.lastExpressionStart);
+				if (lastExpressionStartOffset >= 0) {
+					if (offset < tokenOffset) {
+						// offset is in front of this token
+						return MakeResult (text, lastExpressionStartOffset, tokenOffset, frame.contexts);
+					} else {
+						// offset is IN this token
+						return MakeResult (text, lastExpressionStartOffset, offset, frame.contexts);
+					}
 				} else {
-					// offset is IN this token
-					return MakeResult (text, lastExpressionStartOffset, offset, frame.contexts);
+					return new ExpressionResult (null, frame.contexts);
 				}
-			} else {
-				return new ExpressionResult (null, frame.contexts);
+			} finally {
+				lexer.Dispose ();
 			}
 		}
 
@@ -780,6 +813,9 @@ namespace MonoDevelop.CSharp.Resolver
 					frame = frame.parent;
 				}
 				break;
+			case Tokens.From:
+				frame.SetContext (new ExpressionContext.LinqContext (token.Location.Line - 1, token.Location.Column - 1));
+				break;
 			case Tokens.LessThan:
 				if (Tokens.ValidInsideTypeName[lastToken]) {
 					frame = new Frame (frame, '<');
@@ -832,7 +868,15 @@ namespace MonoDevelop.CSharp.Resolver
 		void TrackCurrentContext (Token token)
 		{
 			if (frame.state == FrameState.ObjectCreation) {
-				if (token.Kind == Tokens.CloseParenthesis) {
+				if (token.Kind == Tokens.CloseSquareBracket) {
+					// handle collection initalizers new string[] { "foo" }.
+					lexer.StartPeek ();
+					if (!(lastToken == Tokens.OpenSquareBracket && lexer.Peek ().Kind == Tokens.OpenCurlyBrace) && frame.context.IsObjectCreation) {
+						frame.SetContext (ExpressionContext.Default);
+						frame.lastExpressionStart = frame.lastNewTokenStart;
+					}
+					// keep frame.state
+				} else if (token.Kind == Tokens.CloseParenthesis || token.Kind == Tokens.CloseCurlyBrace) {
 					if (frame.context.IsObjectCreation) {
 						frame.SetContext (ExpressionContext.Default);
 						frame.lastExpressionStart = frame.lastNewTokenStart;
@@ -1064,7 +1108,7 @@ namespace MonoDevelop.CSharp.Resolver
 			return FindFullExpression (text, offset, methodBody);
 		}
 
-		ExpressionResult FindFullExpression (string text, int offset, Frame initialFrame)
+ 		ExpressionResult FindFullExpression (string text, int offset, Frame initialFrame)
 		{
 			Init (text, offset);
 			
@@ -1117,7 +1161,6 @@ namespace MonoDevelop.CSharp.Resolver
 					if (lastExpressionStartOffset == alternateResultStartOffset && alternateResultStartOffset >= 0)
 						resultStartOffset = lastExpressionStartOffset;
 					if (resultFrame.type == FrameType.Popped || lastExpressionStartOffset != resultStartOffset || token.Kind == Tokens.Dot || token.Kind == Tokens.DoubleColon) {
-						
 						// now we can change the context based on the next token
 						if (frame == resultFrame && Tokens.IdentifierTokens[token.Kind]) {
 							// the expression got aborted because of an identifier. This means the
