@@ -27,15 +27,10 @@
 //
 
 using System;
-using System.Collections.Generic;
 
-using Mono.Cecil;
-
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Projects.Dom;
-using MonoDevelop.Ide.Gui.Pads;
 using MonoDevelop.Ide.Gui.Components;
+using MonoDevelop.Ide.Gui;
 
 namespace MonoDevelop.AssemblyBrowser
 {
