@@ -76,6 +76,8 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			}
 		}
 		
+		public string CustomMessage { get; set; }
+		
 		internal void SetCustomFont (Pango.FontDescription desc)
 		{
 			this.desc = desc;
@@ -85,8 +87,8 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 				headerLayout.FontDescription = desc;
 		}
 		
-		const int spacing = 4;
-		const int categoryHeaderSize = 20;
+		const int spacing = 2;
+		const int categoryHeaderSize = 25;
 		Pango.Layout layout;
 		Pango.Layout headerLayout;
 		
@@ -194,11 +196,14 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			Gdk.Drawable  draw = e.Window;
 			Gdk.Rectangle area = e.Area;
-			if (this.categories.Count == 0) {
+			if (this.categories.Count == 0 || !string.IsNullOrEmpty (CustomMessage)) {
 				Pango.Layout messageLayout = new Pango.Layout (this.PangoContext);
 				messageLayout.Alignment = Pango.Alignment.Center;
 				messageLayout.Width = (int)(Allocation.Width * 2 / 3 * Pango.Scale.PangoScale);
-				messageLayout.SetText (MonoDevelop.Core.GettextCatalog.GetString ("There are no tools available for the current document."));
+				if (!string.IsNullOrEmpty (CustomMessage))
+					messageLayout.SetText (CustomMessage);
+				else
+					messageLayout.SetText (MonoDevelop.Core.GettextCatalog.GetString ("There are no tools available for the current document."));
 				draw.DrawLayout (Style.TextGC (StateType.Normal), Allocation.Width * 1 / 6 , 12, messageLayout);
 				messageLayout.Dispose ();
 				return true;
@@ -228,8 +233,10 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 					cr.RelLineTo (0, -itemDimension.Height);
 					cr.ClosePath ();
 					Cairo.Gradient pat = new Cairo.LinearGradient (xpos, ypos, xpos, ypos + itemDimension.Height);
-					pat.AddColorStop (0, Convert (Style.Mid (StateType.Normal)));
-					pat.AddColorStop (1, Convert (Style.Base (StateType.Normal)));
+					Cairo.Color ccol = Convert (Style.Mid (StateType.Normal));
+					pat.AddColorStop (0, ccol);
+					ccol.A = 0.4;
+					pat.AddColorStop (1, ccol);
 					cr.Pattern = pat;
 					cr.FillPreserve ();
 					
@@ -436,6 +443,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			int xpos = spacing;
 			int ypos = spacing;
 			HideTooltipWindow ();
+			var oldItem = mouseOverItem;
 			mouseOverItem = null;
 			this.mouseX = (int)e.X + (int)(this.hAdjustement != null ? this.hAdjustement.Value : 0);
 			this.mouseY = (int)e.Y + (int)(this.vAdjustement != null ? this.vAdjustement.Value : 0);
@@ -453,7 +461,8 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 				}
 			});
 			
-			this.QueueDraw ();
+			if (oldItem != mouseOverItem)
+				this.QueueDraw ();
 			
 			return base.OnMotionNotifyEvent (e);
 		}
@@ -777,7 +786,6 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			}
 			if (tooltipWindow != null) {
 				tooltipWindow.Destroy ();
-				tooltipWindow.Dispose ();
 				tooltipWindow = null;
 			}
 		}
