@@ -29,27 +29,24 @@
 using System;
 using System.Linq;
 using System.Collections.Generic;
-using System.IO;
-using System.Threading;
 using System.Text.RegularExpressions;
 
 using Gtk;
 using Gdk;
 
-using MonoDevelop.Components;
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Commands;
-using MonoDevelop.Gettext.Editor;
+using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Ide.Tasks;
+using MonoDevelop.Gettext.Editor;
 using Mono.TextEditor;
+using MonoDevelop.Ide.Gui.Components;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.Gettext
 {
 	[System.ComponentModel.Category("widget")]
 	[System.ComponentModel.ToolboxItem(true)]
-	public partial class POEditorWidget : Gtk.Bin
+	public partial class POEditorWidget : Gtk.Bin, IUndoHandler
 	{
 		TranslationProject project;
 		CatalogHeadersWidget headersEditor;
@@ -57,8 +54,10 @@ namespace MonoDevelop.Gettext
 		ListStore foundInStore;
 		Catalog catalog;
 		string  poFileName;
-		Gtk.Tooltips tooltips = new Gtk.Tooltips ();
-
+		Mono.TextEditor.TextEditorOptions options = new Mono.TextEditor.TextEditorOptions ();
+		Mono.TextEditor.TextEditor texteditorOriginal = new Mono.TextEditor.TextEditor ();
+		Mono.TextEditor.TextEditor texteditorPlural = new Mono.TextEditor.TextEditor ();
+		
 		static List<POEditorWidget> widgets = new List<POEditorWidget> (); 
 		
 		public Catalog Catalog {
@@ -70,7 +69,7 @@ namespace MonoDevelop.Gettext
 				headersEditor.CatalogHeaders = catalog;
 				ClearTextview ();
 				AddTextview (0);
-				this.GetTextView (0).Buffer.Changed += delegate {
+				this.GetTextView (0).Document.TextReplaced += delegate {
 					Gtk.TreeIter iter = SelectedIter;
 					if (treeviewEntries.Selection.IterIsSelected (iter)) {
 						store.SetValue (iter, (int)Columns.Stock, GetStockForEntry (SelectedEntry));
@@ -98,7 +97,7 @@ namespace MonoDevelop.Gettext
 		public POEditorWidget (TranslationProject project)
 		{
 			this.project = project;
-			this.Build();
+			this.Build ();
 			this.headersEditor = new CatalogHeadersWidget ();
 			this.notebookPages.AppendPage (headersEditor, new Gtk.Label ());
 			
@@ -106,16 +105,16 @@ namespace MonoDevelop.Gettext
 			AddButton (GettextCatalog.GetString ("Headers")).Active = false;
 			
 			// entries tree view 
-			store = new ListStore (typeof (string), typeof (bool), typeof (string), typeof (string), typeof (CatalogEntry), typeof (Gdk.Color), typeof(int), typeof (Gdk.Color));
+			store = new ListStore (typeof(string), typeof(bool), typeof(string), typeof(string), typeof(CatalogEntry), typeof(Gdk.Color), typeof(int), typeof(Gdk.Color));
 			this.treeviewEntries.Model = store;
 			
-			treeviewEntries.AppendColumn (String.Empty, new CellRendererPixbuf (), "stock_id", Columns.Stock, "cell-background-gdk", Columns.RowColor);
+			treeviewEntries.AppendColumn (String.Empty, new CellRendererIcon (), "stock_id", Columns.Stock, "cell-background-gdk", Columns.RowColor);
 			
 			CellRendererToggle cellRendFuzzy = new CellRendererToggle ();
 			cellRendFuzzy.Toggled += new ToggledHandler (FuzzyToggled);
 			cellRendFuzzy.Activatable = true;
 			treeviewEntries.AppendColumn (GettextCatalog.GetString ("Fuzzy"), cellRendFuzzy, "active", Columns.Fuzzy, "cell-background-gdk", Columns.RowColor);
-			 
+			
 			CellRendererText original = new CellRendererText ();
 			original.Ellipsize = Pango.EllipsizeMode.End;
 			treeviewEntries.AppendColumn (GettextCatalog.GetString ("Original string"), original, "text", Columns.String, "cell-background-gdk", Columns.RowColor, "foreground-gdk", Columns.ForeColor);
@@ -141,47 +140,57 @@ namespace MonoDevelop.Gettext
 			treeviewEntries.GetColumn (3).Resizable = true;
 			treeviewEntries.GetColumn (3).Expand = true;
 			// found in tree view
-			foundInStore = new ListStore (typeof (string), typeof (string), typeof (string));
+			foundInStore = new ListStore (typeof(string), typeof(string), typeof(string), typeof(Pixbuf));
 			this.treeviewFoundIn.Model = foundInStore;
 			
-			treeviewFoundIn.AppendColumn ("", new CellRendererText (), "text", FoundInColumns.File);
+			TreeViewColumn fileColumn = new TreeViewColumn ();
+			var pixbufRenderer = new CellRendererPixbuf ();
+			fileColumn.PackStart (pixbufRenderer, false);
+			fileColumn.SetAttributes (pixbufRenderer, "pixbuf", FoundInColumns.Pixbuf);
+			
+			CellRendererText textRenderer = new CellRendererText ();
+			fileColumn.PackStart (textRenderer, true);
+			fileColumn.SetAttributes (textRenderer, "text", FoundInColumns.File);
+			treeviewFoundIn.AppendColumn (fileColumn);
+			
 			treeviewFoundIn.AppendColumn ("", new CellRendererText (), "text", FoundInColumns.Line);
 			treeviewFoundIn.HeadersVisible = false;
 			treeviewFoundIn.GetColumn (1).FixedWidth = 100;
 			
-			treeviewFoundIn.RowActivated += delegate (object sender, RowActivatedArgs e) {
+			treeviewFoundIn.RowActivated += delegate(object sender, RowActivatedArgs e) {
 				Gtk.TreeIter iter;
-				foundInStore.GetIter (out iter, e.Path); 				
+				foundInStore.GetIter (out iter, e.Path);
 				string line = foundInStore.GetValue (iter, (int)FoundInColumns.Line) as string;
 				string file = foundInStore.GetValue (iter, (int)FoundInColumns.FullFileName) as string;
 				int lineNr = 1;
 				try {
 					lineNr = 1 + int.Parse (line);
-				} catch {}
-				MonoDevelop.Ide.Gui.IdeApp.Workbench.OpenDocument (file, lineNr, 1, true);
+				} catch {
+				}
+				IdeApp.Workbench.OpenDocument (file, lineNr, 1, true);
 			};
 			this.notebookTranslated.RemovePage (0);
-			this.entryFilter.Text = "";
-			entryFilter.Changed += delegate {
+			this.searchEntryFilter.Entry.Text = "";
+			searchEntryFilter.Entry.Changed += delegate {
 				UpdateFromCatalog ();
 			};
 			
 			this.togglebuttonFuzzy.Active = PropertyService.Get ("Gettext.ShowFuzzy", true);
-			tooltips.SetTip (this.togglebuttonFuzzy, GettextCatalog.GetString ("Show fuzzy translations"), null);
+			this.togglebuttonFuzzy.TooltipText = GettextCatalog.GetString ("Show fuzzy translations");
 			this.togglebuttonFuzzy.Toggled += delegate {
 				MonoDevelop.Core.PropertyService.Set ("Gettext.ShowFuzzy", this.togglebuttonFuzzy.Active);
 				UpdateFromCatalog ();
 			};
 			
 			this.togglebuttonMissing.Active = PropertyService.Get ("Gettext.ShowMissing", true);
-			tooltips.SetTip (this.togglebuttonMissing, GettextCatalog.GetString ("Show missing translations"), null);
+			this.togglebuttonMissing.TooltipText = GettextCatalog.GetString ("Show missing translations");
 			this.togglebuttonMissing.Toggled += delegate {
 				MonoDevelop.Core.PropertyService.Set ("Gettext.ShowMissing", this.togglebuttonMissing.Active);
 				UpdateFromCatalog ();
 			};
 			
 			this.togglebuttonOk.Active = PropertyService.Get ("Gettext.ShowTranslated", true);
-			tooltips.SetTip (this.togglebuttonOk, GettextCatalog.GetString ("Show valid translations"), null);
+			this.togglebuttonOk.TooltipText = GettextCatalog.GetString ("Show valid translations");
 			this.togglebuttonOk.Toggled += delegate {
 				MonoDevelop.Core.PropertyService.Set ("Gettext.ShowTranslated", this.togglebuttonOk.Active);
 				UpdateFromCatalog ();
@@ -191,7 +200,7 @@ namespace MonoDevelop.Gettext
 				if (this.isUpdating)
 					return;
 				if (this.currentEntry != null) {
-					string[] lines = StringEscaping.FromGettextFormat (textviewComments.Buffer.Text).Split (new string[] { System.Environment.NewLine }, System.StringSplitOptions.None);
+					string[] lines = StringEscaping.FromGettextFormat (textviewComments.Buffer.Text).Split (new string[] { System.Environment.NewLine }, System.StringSplitOptions.None);
 					for (int i = 0; i < lines.Length; i++) {
 						if (!lines[i].StartsWith ("#"))
 							lines[i] = "# " + lines[i];
@@ -204,32 +213,71 @@ namespace MonoDevelop.Gettext
 				ShowPopup ();
 			};
 			
-			this.treeviewEntries.ButtonReleaseEvent += delegate (object sender, Gtk.ButtonReleaseEventArgs e) {
+			this.treeviewEntries.ButtonReleaseEvent += delegate(object sender, Gtk.ButtonReleaseEventArgs e) {
 				if (e.Event.Button == 3)
 					ShowPopup ();
 			};
-			this.buttonOptions.Label = GettextCatalog.GetString ("Options");
-			this.buttonOptions.StockImage = Gtk.Stock.Properties;
-			this.buttonOptions.MenuCreator = CreateOptionsMenu;
+			
+			searchEntryFilter.Ready = true;
+			searchEntryFilter.Visible = true;
+			searchEntryFilter.ForceFilterButtonVisible = true;
+			searchEntryFilter.RequestMenu += delegate {
+				searchEntryFilter.Menu = CreateOptionsMenu ();
+			};
+		
+//			this.buttonOptions.Label = GettextCatalog.GetString ("Options");
+			//			this.buttonOptions.StockImage = Gtk.Stock.Properties;
+			//			this.buttonOptions.MenuCreator = ;
 			widgets.Add (this);
 			UpdateTasks ();
-//			this.vpaned2.AcceptPosition += delegate {
-//				PropertyService.Set ("Gettext.SplitPosition", vpaned2.Position / (double)Allocation.Height);
-//				inMove = false;
-//			};
-//			this.vpaned2.CancelPosition += delegate {
-//				inMove = false;
-//			};
-//			this.vpaned2.MoveHandle += delegate {
-//				inMove = true;
-//			};
-//			this.ResizeChecked += delegate {
-//				if (inMove)
-//					return;
-//				int newPosition = (int)(Allocation.Height * PropertyService.Get ("Gettext.SplitPosition", 0.3d));
-//				if (vpaned2.Position != newPosition)
-//					vpaned2.Position = newPosition;
-//			};
+			//			this.vpaned2.AcceptPosition += delegate {
+			//				PropertyService.Set ("Gettext.SplitPosition", vpaned2.Position / (double)Allocation.Height);
+			//				inMove = false;
+			//			};
+			//			this.vpaned2.CancelPosition += delegate {
+			//				inMove = false;
+			//			};
+			//			this.vpaned2.MoveHandle += delegate {
+			//				inMove = true;
+			//			};
+			//			this.ResizeChecked += delegate {
+			//				if (inMove)
+			//					return;
+			//				int newPosition = (int)(Allocation.Height * PropertyService.Get ("Gettext.SplitPosition", 0.3d));
+			//				if (vpaned2.Position != newPosition)
+			//					vpaned2.Position = newPosition;
+			//			};
+			checkbuttonWhiteSpaces.Toggled += CheckbuttonWhiteSpacesToggled;
+			options.ShowLineNumberMargin = false;
+			options.ShowFoldMargin = false;
+			options.ShowIconMargin = false;
+			options.ShowInvalidLines = false;
+			options.ShowSpaces = options.ShowTabs = options.ShowEolMarkers = false;
+			options.ColorScheme = PropertyService.Get ("ColorScheme", "Default");
+			options.FontName = PropertyService.Get<string> ("FontName");
+			
+			this.scrolledwindowOriginal.Child = this.texteditorOriginal;
+			this.scrolledwindowPlural.Child = this.texteditorPlural;
+			this.texteditorOriginal.Show ();
+			this.texteditorPlural.Show ();
+			texteditorOriginal.ModifyBase (Gtk.StateType.Normal, Style.Base (Gtk.StateType.Insensitive));
+			texteditorPlural.ModifyBase (Gtk.StateType.Normal, Style.Base (Gtk.StateType.Insensitive));
+			this.texteditorOriginal.Options = options;
+			this.texteditorPlural.Options = options;
+			this.texteditorOriginal.Document.ReadOnly = true;
+			this.texteditorPlural.Document.ReadOnly = true;
+		}
+		
+		void CheckbuttonWhiteSpacesToggled (object sender, EventArgs e)
+		{
+			options.ShowSpaces = options.ShowTabs = checkbuttonWhiteSpaces.Active;
+			texteditorOriginal.QueueDraw ();
+			texteditorPlural.QueueDraw ();
+			for (int i = this.notebookTranslated.NPages - 1; i >= 0; i--) {
+				Mono.TextEditor.TextEditor view = GetTextView (i);
+				if (view != null)
+					view.QueueDraw ();
+			}
 		}
 		
 		#region Options
@@ -293,7 +341,7 @@ namespace MonoDevelop.Gettext
 		}
 		#endregion
 		
-		public Menu CreateOptionsMenu (MenuButton button)
+		public Menu CreateOptionsMenu ()
 		{
 			Menu menu = new Menu ();
 			
@@ -331,21 +379,21 @@ namespace MonoDevelop.Gettext
 				if (DoSearchIn != SearchIn.Both) {
 					DoSearchIn = SearchIn.Both;
 					UpdateFromCatalog ();
-					menu.Dispose ();
+					menu.Destroy ();
 				}
 			};
 			original.Activated += delegate {
 				if (DoSearchIn != SearchIn.Original) {
 					DoSearchIn = SearchIn.Original;
 					UpdateFromCatalog ();
-					menu.Dispose ();
+					menu.Destroy ();
 				}
 			};
 			translated.Activated += delegate {
 				if (DoSearchIn != SearchIn.Translated) {
 					DoSearchIn = SearchIn.Translated;
 					UpdateFromCatalog ();
-					menu.Dispose ();
+					menu.Destroy ();
 				}
 			};
 			
@@ -397,11 +445,11 @@ namespace MonoDevelop.Gettext
 			this.Catalog = newCatalog;
 		}
 		
-		TextView GetTextView (int index)
+		Mono.TextEditor.TextEditor GetTextView (int index)
 		{
 			ScrolledWindow window = this.notebookTranslated.GetNthPage (index) as ScrolledWindow;
 			if (window != null)
-				return window.Child as TextView;
+				return window.Child as Mono.TextEditor.TextEditor;
 			return null;
 		}
 		
@@ -414,19 +462,23 @@ namespace MonoDevelop.Gettext
 		void AddTextview (int index)
 		{
 			ScrolledWindow window = new ScrolledWindow ();
-			TextView textView = new TextView ();
+			Mono.TextEditor.TextEditor textView = new Mono.TextEditor.TextEditor ();
 			window.Child = textView;
-			textView.AcceptsTab = false;	
-			textView.Buffer.Changed += delegate {
+			textView.Options = options;
+			textView.Document.TextReplaced += delegate {
 				if (this.isUpdating)
 					return;
 				try {
-					if (this.currentEntry != null)
-						this.currentEntry.SetTranslation (StringEscaping.FromGettextFormat (textView.Buffer.Text), index);
-					MonoDevelop.Ide.Gui.IdeApp.Workbench.StatusBar.ShowReady ();
+					if (this.currentEntry != null) {
+						string escapedText = StringEscaping.FromGettextFormat (textView.Document.Text);
+						string oldText     = this.currentEntry.GetTranslation (index);
+						this.currentEntry.SetTranslation (escapedText, index);
+						AddChange (this.currentEntry, oldText, escapedText, index);
+					}
+					IdeApp.Workbench.StatusBar.ShowReady ();
 					textView.ModifyBase (Gtk.StateType.Normal, Style.Base (Gtk.StateType.Normal));
 				} catch (System.Exception e) {
-					MonoDevelop.Ide.Gui.IdeApp.Workbench.StatusBar.ShowError (e.Message);
+					IdeApp.Workbench.StatusBar.ShowError (e.Message);
 					textView.ModifyBase (Gtk.StateType.Normal, errorColor);
 				}
 				UpdateProgressBar ();
@@ -478,7 +530,7 @@ namespace MonoDevelop.Gettext
 		
 		void RemoveEntry (CatalogEntry entry)
 		{
-			bool yes = MonoDevelop.Core.Gui.MessageService.AskQuestion (GettextCatalog.GetString ("Do you really want to remove the translation string {0} (It will be removed from all translations)?", entry.String),
+			bool yes = MessageService.AskQuestion (GettextCatalog.GetString ("Do you really want to remove the translation string {0} (It will be removed from all translations)?", entry.String),
 			                                                            AlertButton.Cancel, AlertButton.Remove) == AlertButton.Remove;
 
 			if (yes) {
@@ -497,20 +549,21 @@ namespace MonoDevelop.Gettext
 			catalog.GetStatistics (out all, out fuzzy, out missing, out bad, out untrans);
 			double percentage = all > 0 ? ((double)(all - untrans) / all) * 100 : 0.0;
 			string barText = String.Format (GettextCatalog.GetString ("{0:#00.00}% Translated"), percentage);
-			if (missing > 0 || fuzzy > 0)
+			if (untrans > 0 || fuzzy > 0)
 				barText += " (";
-			
-			if (fuzzy > 0) {
-				barText += String.Format (GettextCatalog.GetPluralString ("{0} Fuzzy Message", "{0} Fuzzy Messages", fuzzy), fuzzy);
+
+			if (untrans > 0) {
+				barText += String.Format (GettextCatalog.GetPluralString ("{0} Missing Message", "{0} Missing Messages", untrans), untrans);
 			}
-			
-			if (missing > 0) {
-				if (fuzzy > 0) {
+
+			if (fuzzy > 0) {
+				if (untrans > 0) {
 					barText += ", ";
 				}
-				barText += String.Format (GettextCatalog.GetPluralString ("{0} Missing Message", "{0} Missing Messages", missing), missing);
+				barText += String.Format (GettextCatalog.GetPluralString ("{0} Fuzzy Message", "{0} Fuzzy Messages", fuzzy), fuzzy);
 			}
-			if (missing > 0 || fuzzy > 0)
+
+			if (untrans > 0 || fuzzy > 0)
 				barText += ")";
 			
 			this.progressbar1.Text = barText;
@@ -520,17 +573,17 @@ namespace MonoDevelop.Gettext
 		
 		#region EntryEditor handling
 		CatalogEntry currentEntry;
-		Dictionary<TextView, bool> gtkSpellSet = new Dictionary<TextView, bool> (); 
+//		Dictionary<Mono.TextEditor.TextEditor, bool> gtkSpellSet = new Dictionary<Mono.TextEditor.TextEditor, bool> (); 
 		void RemoveTextViewsFrom (int index)
 		{
 			for (int i = this.notebookTranslated.NPages - 1; i >= index; i--) {
-				TextView view = GetTextView (i);
+				Mono.TextEditor.TextEditor view = GetTextView (i);
 				if (view == null)
 					continue;
-				if (gtkSpellSet.ContainsKey (view)) {
+//				if (gtkSpellSet.ContainsKey (view)) {
 //					GtkSpell.Detach (view);
-					gtkSpellSet.Remove (view);
-				}
+//					gtkSpellSet.Remove (view);
+//				}
 				this.notebookTranslated.RemovePage (i);
 			}
 		}
@@ -540,8 +593,9 @@ namespace MonoDevelop.Gettext
 			this.isUpdating = true;
 			try {
 				currentEntry = entry;
-					
-				this.labelOriginal.Text = entry != null ? StringEscaping.ToGettextFormat (entry.String) : "";
+				this.texteditorOriginal.Caret.Offset = 0;
+				this.texteditorOriginal.Document.Text = entry != null ? StringEscaping.ToGettextFormat (entry.String) : "";
+				this.texteditorOriginal.VAdjustment.Value = this.texteditorOriginal.HAdjustment.Value = 0;
 				
 //				if (GtkSpell.IsSupported && !gtkSpellSet.ContainsKey (this.textviewOriginal)) {
 //					GtkSpell.Attach (this.textviewOriginal, "en");
@@ -552,7 +606,9 @@ namespace MonoDevelop.Gettext
 				this.notebookTranslated.ShowTabs = entry != null && entry.HasPlural;
 				
 				if (entry != null && entry.HasPlural) {
-					this.labelPlural.Text = StringEscaping.ToGettextFormat (entry.PluralString);
+					this.texteditorPlural.Caret.Offset = 0;
+					this.texteditorPlural.Document.Text = StringEscaping.ToGettextFormat (entry.PluralString);
+					this.texteditorPlural.VAdjustment.Value = this.texteditorPlural.HAdjustment.Value = 0;
 //					if (GtkSpell.IsSupported && !gtkSpellSet.ContainsKey (this.textviewOriginalPlural)) {
 //						GtkSpell.Attach (this.textviewOriginalPlural, "en");
 //						this.gtkSpellSet[this.textviewOriginalPlural] = true;
@@ -569,14 +625,18 @@ namespace MonoDevelop.Gettext
 					}
 					
 					for (int i = 0; i < entry.NumberOfTranslations; i++) {
-						TextView textView = GetTextView (i);
+						Mono.TextEditor.TextEditor textView = GetTextView (i);
 						if (textView == null)
 							continue;
-						textView.Buffer.Text = entry != null ? StringEscaping.ToGettextFormat (entry.GetTranslation (i)) : "";
-						if (GtkSpell.IsSupported && !gtkSpellSet.ContainsKey (textView)) {
+						textView.ClearSelection ();
+						textView.Document.Text = entry != null ? StringEscaping.ToGettextFormat (entry.GetTranslation (i)) : "";
+						textView.Caret.Offset = textView.Document.Text.Length;
+						textView.VAdjustment.Value = textView.HAdjustment.Value = 0;
+						textView.Document.CommitUpdateAll ();
+/*						if (GtkSpell.IsSupported && !gtkSpellSet.ContainsKey (textView)) {
 							GtkSpell.Attach (textView, "en");
 							this.gtkSpellSet[textView] = true;
-						}
+						}*/
 					}
 					
 					foreach (string reference in entry.References) {
@@ -591,16 +651,16 @@ namespace MonoDevelop.Gettext
 							line = "?";
 						}
 						string fullName = System.IO.Path.Combine (System.IO.Path.GetDirectoryName (this.poFileName), file);
-						this.foundInStore.AppendValues (file, line, fullName);
+						this.foundInStore.AppendValues (file, line, fullName, DesktopService.GetPixbufForFile (fullName, IconSize.Menu));
 					}
 				}
 				
 				this.textviewComments.Buffer.Text = entry != null ? StringEscaping.ToGettextFormat (entry.Comment) : null;
 				
-				if (GtkSpell.IsSupported) {
+/*				if (GtkSpell.IsSupported) {
 					foreach (TextView view in this.gtkSpellSet.Keys)
 						GtkSpell.Recheck (view);
-				}
+				}*/
 			} finally {
 				this.isUpdating = false;
 			}
@@ -624,7 +684,8 @@ namespace MonoDevelop.Gettext
 		{
 			File,
 			Line,
-			FullFileName
+			FullFileName,
+			Pixbuf
 		}
 		
 		protected override void OnStyleSet (Gtk.Style previous_style)
@@ -665,10 +726,14 @@ namespace MonoDevelop.Gettext
 		{
 			if (entry.References.Length == 0)
 				return Gtk.Stock.DialogError;
-			return entry.IsFuzzy ? Gtk.Stock.About : entry.IsTranslated ? Gtk.Stock.Apply : Gtk.Stock.Cancel;
+			return entry.IsFuzzy ? iconFuzzy : entry.IsTranslated ? iconValid : iconMissing;
 		}
 		
-		static Color translated   = new Color (255, 255, 255);
+		static string iconFuzzy   = "silk_error";// "md-translation-fuzzy";
+		static string iconValid   = "silk_accept";//"md-translation-valid";
+		static string iconMissing = "silk_exclamation";//"md-translation-missing";
+		
+//		static Color translated   = new Color (255, 255, 255);
 		static Color untranslated = new Color (234, 232, 227);
 		static Color fuzzy        = new Color (237, 226, 187);
 		static Color missing      = new Color (237, 167, 167);
@@ -704,6 +769,8 @@ namespace MonoDevelop.Gettext
 		CatalogEntry SelectedEntry {
 			get {
 				TreeIter iter = SelectedIter;
+				if (iter.Equals (Gtk.TreeIter.Zero))
+					return null;
 				if (treeviewEntries.Selection.IterIsSelected (iter))
 					return store.GetValue (iter, (int)Columns.CatalogEntry) as CatalogEntry;
 				return null;
@@ -788,16 +855,13 @@ namespace MonoDevelop.Gettext
 			return true;
 		}
 		
-		Thread updateThread = null;
-		bool updateIsRunning = false;
+		FilterWorkerThread updateThread = null;
 		string filter = "";
 		Regex  regex = new Regex ("");
 		
 		void UpdateFromCatalog ()
 		{
-			if (updateIsRunning)
-				updateIsRunning = false;
-			filter = this.entryFilter.Text;
+			filter = this.searchEntryFilter.Entry.Text;
 			if (!IsCaseSensitive && filter != null)
 				filter = filter.ToUpper ();
 			if (RegexSearch) {
@@ -808,22 +872,41 @@ namespace MonoDevelop.Gettext
 					regex = new Regex (filter, options);
 				} catch (Exception e) {
 					IdeApp.Workbench.StatusBar.ShowError (e.Message);
-					this.entryFilter.ModifyBase (StateType.Normal, errorColor);
+					this.searchEntryFilter.Entry.ModifyBase (StateType.Normal, errorColor);
 					return;
 				}
 			}
-			this.entryFilter.ModifyBase (StateType.Normal, Style.Base (StateType.Normal));
-			IdeApp.Workbench.StatusBar.BeginProgress (GettextCatalog.GetString ("Update catalog list..."));
-			updateThread = new Thread (UpdateWorkerThread);
-			updateThread.IsBackground = true;
-			updateThread.Priority = ThreadPriority.Lowest;
+			this.searchEntryFilter.Entry.ModifyBase (StateType.Normal, Style.Base (StateType.Normal));
+			StopFilterWorkerThread ();
+			updateThread = new FilterWorkerThread (this);
 			updateThread.Start ();
 		}
 		
-		
+		void StopFilterWorkerThread ()
+		{
+			if (updateThread != null)  {
+				updateThread.Stop ();
+				//updateThread.WaitForFinish ();
+				updateThread = null;
+			}
+		}
+		bool IsVisible (TreePath path)
+		{
+			TreePath start, end, cur;
+			this.treeviewEntries.GetVisibleRange (out start, out end);
+			TreeIter iter;
+			if (!store.GetIter (out iter, start))
+				return false;
+			do {
+				cur = store.GetPath (iter);
+				if (cur.Equals (path))
+					return true;
+			} while (!cur.Equals (end) && store.IterNext (ref iter));
+			return false;
+		}
 		public void SelectEntry (CatalogEntry entry)
 		{
-			if (updateIsRunning)
+			if (updateThread != null && !updateThread.IsStopped)
 				return;
 			
 			TreeIter iter;
@@ -832,7 +915,9 @@ namespace MonoDevelop.Gettext
 					CatalogEntry curEntry = store.GetValue (iter, 4) as CatalogEntry;
 					if (entry == curEntry) {
 						this.treeviewEntries.Selection.SelectIter (iter);
-						this.treeviewEntries.ScrollToCell (store.GetPath (iter), treeviewEntries.GetColumn (0), true, 0, 0);
+						TreePath iterPath = store.GetPath (iter);
+						if (!IsVisible (iterPath))
+							this.treeviewEntries.ScrollToCell (iterPath, treeviewEntries.GetColumn (0), true, 0, 0);
 						return;
 					}
 				} while (store.IterNext (ref iter));
@@ -849,58 +934,68 @@ namespace MonoDevelop.Gettext
 			SelectEntry (entry);
 		}
 		
-		public void UpdateWorkerThread ()
+		class FilterWorkerThread : WorkerThread
 		{
-			int number = 1;
-			double count = catalog.Count;
-			List<CatalogEntry> foundEntries = new List<CatalogEntry> ();
-			updateIsRunning = true;
-			try {
-				foreach (CatalogEntry curEntry in catalog) {
-					if (!updateIsRunning)
-						break;
-					number++;
-					if (number % 50 == 0) {
-						DispatchService.GuiSyncDispatch (delegate {
-							if (number < 60)
-								store.Clear ();
-							MonoDevelop.Ide.Gui.IdeApp.Workbench.StatusBar.SetProgressFraction (Math.Min (1.0,  Math.Max (0.0, number / (double)count)));
-							foreach (CatalogEntry entry in foundEntries) {
-								if (!updateIsRunning)
-									break;
-								
-								store.AppendValues (GetStockForEntry (entry), 
-										            entry.IsFuzzy,
-										            StringEscaping.ToGettextFormat (entry.String), 
-										            StringEscaping.ToGettextFormat (entry.GetTranslation (0)), 
-										            entry,
-										            GetRowColorForEntry (entry),
-								                    GetTypeSortIndicator (entry),
-								                    GetForeColorForEntry (entry)
-								                    );
-							}
-						});
-						foundEntries.Clear ();
-					}
-					if (!ShouldFilter (curEntry, filter)) 
-						foundEntries.Add (curEntry);
-				}
-			} catch (Exception) {
+			POEditorWidget widget;
+			
+			public FilterWorkerThread (POEditorWidget widget)
+			{
+				this.widget = widget;
+			}
+			
+			protected override void InnerRun ()
+			{
+				int number = 1, found = 0;
+				double count = widget.catalog.Count;
+				ListStore newStore = new ListStore (typeof(string), typeof(bool), typeof(string), typeof(string), typeof(CatalogEntry), typeof(Gdk.Color), typeof(int), typeof(Gdk.Color));
+				StatusBarContext statusBar = null;
+				DispatchService.GuiSyncDispatch (delegate {
+					statusBar = IdeApp.Workbench.StatusBar.CreateContext ();
+					statusBar.BeginProgress (GettextCatalog.GetString ("Update catalog list..."));
+				});
 				
-			} finally {
-				if (updateIsRunning) {
-					MonoDevelop.Core.Gui.DispatchService.GuiSyncDispatch (delegate {
-						if (number < 60)
-							store.Clear ();
-						foreach (CatalogEntry entry in foundEntries) {
-							store.AppendValues (GetStockForEntry (entry), entry.IsFuzzy, StringEscaping.ToGettextFormat (entry.String), StringEscaping.ToGettextFormat (entry.GetTranslation (0)), entry, GetRowColorForEntry (entry), GetTypeSortIndicator (entry), GetForeColorForEntry (entry));
+				try {
+					foreach (CatalogEntry entry in widget.catalog) {
+						if (IsStopping)
+							return;
+						number++;
+						if (number % 50 == 0) {
+							DispatchService.GuiSyncDispatch (delegate {
+								statusBar.SetProgressFraction (Math.Min (1.0, Math.Max (0.0, number / (double)count)));
+							});
 						}
+						if (!widget.ShouldFilter (entry, widget.filter)) {
+							newStore.AppendValues (GetStockForEntry (entry), 
+								entry.IsFuzzy, StringEscaping.ToGettextFormat (entry.String), 
+								StringEscaping.ToGettextFormat (entry.GetTranslation (0)), 
+								entry,
+								widget.GetRowColorForEntry (entry),
+								GetTypeSortIndicator (entry),
+								widget.GetForeColorForEntry (entry)
+							);
+							found++;
+						}
+					}
+				} catch (Exception) {
+				
+				}
+				if (!IsStopping) {
+					DispatchService.GuiSyncDispatch (delegate {
+						widget.store.Dispose ();
+						widget.treeviewEntries.Model = widget.store = newStore;
+						statusBar.EndProgress ();
+						IdeApp.Workbench.StatusBar.ShowMessage (string.Format (GettextCatalog.GetPluralString ("Found {0} catalog entry.", "Found {0} catalog entries.", found), found));
+					});
+				} /*else {
+					MonoDevelop.Core.Gui.DispatchService.GuiSyncDispatch (delegate {
 						MonoDevelop.Ide.Gui.IdeApp.Workbench.StatusBar.EndProgress ();
 					});
-					updateIsRunning = false;
-				}
+				}*/
+				Stop ();
 			}
+
 		}
+		
 #endregion
 		
 #region Toolbar handling
@@ -947,14 +1042,9 @@ namespace MonoDevelop.Gettext
 		
 		protected override void OnDestroyed ()
 		{
-			updateIsRunning = false;
+			StopFilterWorkerThread ();
 			StopTaskWorkerThread ();
 		
-			if (tooltips != null) {
-				tooltips.Destroy ();
-				tooltips = null;
-			}
-			
 			if (store != null) {
 				store.Dispose ();
 				store = null;
@@ -978,7 +1068,7 @@ namespace MonoDevelop.Gettext
 			
 			public TranslationTask (POEditorWidget widget, CatalogEntry entry, string description) : base (widget.poFileName,
 			                                                                           description, 0, 0,
-			                                                                           TaskType.Error)
+			                                                                           TaskSeverity.Error, TaskPriority.Normal, null, widget)
 			{
 				this.widget = widget;
 				this.entry  = entry;
@@ -992,11 +1082,9 @@ namespace MonoDevelop.Gettext
 		
 		void ClearTasks ()
 		{
-			IdeApp.Services.TaskService.ClearExceptCommentTasks ();
-			/*foreach (TranslationTask task in tasks) {
-				IdeApp.Services.TaskService.Remove (task);
-			}*/
+			TaskService.Errors.ClearByOwner (this);
 		}
+		
 		static bool CompareTasks (List<Task> list1, List<Task> list2)
 		{
 			if (list1.Count != list2.Count)
@@ -1011,13 +1099,15 @@ namespace MonoDevelop.Gettext
 			new PointRuleCatalogEntryRule (),
 			new CaseMismatchCatalogEntryRule (),
 			new UnderscoreCatalogEntryRule (),
-			new StringFormatCatalogEntryRule ()
+			new StringFormatCatalogEntryRule (),
+			new EndsWithWhitespaceCatalogEntryRule ()
 		};
 		IEnumerable<CatalogEntryRule> rules = new CatalogEntryRule[] {};
 		
 		public void UpdateRules (string country)
 		{
 			rules = from n in allRules where n.IsValid (country) select n;
+			UpdateTasks ();
 		}
 		
 		abstract class CatalogEntryRule
@@ -1044,6 +1134,19 @@ namespace MonoDevelop.Gettext
 			public abstract string FailReason (CatalogEntry entry);
 		}
 		
+		class EndsWithWhitespaceCatalogEntryRule : CatalogEntryRule
+		{
+			public override bool EntryFails (CatalogEntry entry)
+			{
+				return entry.String.EndsWith (" ") && !entry.GetTranslation (0).EndsWith (" ");
+			}
+			
+			public override string FailReason (CatalogEntry entry)
+			{
+				return GettextCatalog.GetString ("Translation for '{0}' doesn't end with whitespace ' '.", entry.String);
+			}
+		}
+			
 		class PointRuleCatalogEntryRule : CatalogEntryRule
 		{
 			public override bool EntryFails (CatalogEntry entry)
@@ -1075,12 +1178,16 @@ namespace MonoDevelop.Gettext
 		{
 			public override bool EntryFails (CatalogEntry entry)
 			{
-				return entry.String.Contains ("_") && !entry.GetTranslation (0).Contains ("_");
+				return entry.String.Contains ("_") && !entry.GetTranslation (0).Contains ("_") ||
+					!entry.String.Contains ("_") && entry.GetTranslation (0).Contains ("_");
+				
 			}
 			
 			public override string FailReason (CatalogEntry entry)
 			{
-				return GettextCatalog.GetString ("Original string '{0}' contains '_', translation doesn't.", entry.String);
+				if (entry.String.Contains ("_") && !entry.GetTranslation (0).Contains ("_"))
+					return GettextCatalog.GetString ("Original string '{0}' contains '_', translation doesn't.", entry.String);
+				return GettextCatalog.GetString ("Original string '{0}' doesn't contain '_', translation does.", entry.String);
 			}
 		}
 		
@@ -1146,7 +1253,7 @@ namespace MonoDevelop.Gettext
 				if (!CompareTasks (tasks, widget.currentTasks)) {
 					widget.ClearTasks ();
 					widget.currentTasks = tasks;
-					IdeApp.Services.TaskService.AddRange (tasks);
+					TaskService.Errors.AddRange (tasks);
 				}
 				Stop ();
 			}
@@ -1170,5 +1277,101 @@ namespace MonoDevelop.Gettext
 			updateTaskThread.Start ();
 		}
 #endregion
+
+		#region IUndoHandler implementation
+		Stack<Change> undoStack = new Stack<Change> ();
+		Stack<Change> redoStack = new Stack<Change> ();
+		public class Change
+		{
+			POEditorWidget widget;
+			public CatalogEntry Entry {
+				get; 
+				set;
+			}
+			public string OldText {
+				get;
+				set;
+			}
+			public string Text {
+				get;
+				set;
+			}
+			public int Index {
+				get;
+				set;
+			}
+			public Change (POEditorWidget widget, CatalogEntry entry, string oldText, string text, int index)
+			{
+				this.widget = widget;
+				this.Entry = entry;
+				this.OldText = oldText;
+				this.Text  = text;
+				this.Index = index;
+			}
+			
+			public void Undo ()
+			{
+				widget.inUndoOperation = true;
+				widget.SelectEntry (Entry);
+				Mono.TextEditor.TextEditor textView = widget.GetTextView (Index);
+				if (textView != null)
+					textView.Document.Text = OldText;
+				widget.inUndoOperation = false;
+			}
+			
+			public void Redo ()
+			{
+				widget.inUndoOperation = true;
+				widget.SelectEntry (Entry);
+				Mono.TextEditor.TextEditor textView = widget.GetTextView (Index);
+				if (textView != null)
+					textView.Document.Text = Text;
+				widget.inUndoOperation = false;
+			}
+		}
+		
+		bool inUndoOperation = false;
+		public void AddChange (CatalogEntry entry, string oldText, string text, int index)
+		{
+			if (inUndoOperation)
+				return;
+			redoStack.Clear ();
+			undoStack.Push (new Change (this, entry, oldText, text, index));
+		}
+		
+		public void Undo ()
+		{
+			Change change = undoStack.Pop ();
+			change.Undo ();
+			redoStack.Push (change);
+		}
+		
+		public void Redo ()
+		{
+			Change change = redoStack.Pop ();
+			change.Redo ();
+			undoStack.Push (change);
+		}
+		
+		public void BeginAtomicUndo ()
+		{
+		}
+		
+		public void EndAtomicUndo ()
+		{
+		}
+		
+		public bool EnableUndo {
+			get {
+				return undoStack.Count > 0;
+			}
+		}
+		
+		public bool EnableRedo {
+			get {
+				return redoStack.Count > 0;
+			}
+		}
+		#endregion
 	}
 }
