@@ -28,29 +28,25 @@
 
 using System;
 using System.IO;
-using System.Reflection;
 using System.Collections;
 using System.CodeDom;
 using System.CodeDom.Compiler;
 
 using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Dom.Parser;
-using MonoDevelop.Projects.CodeGeneration;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Deployment;
-using Mono.Cecil;
+using MonoDevelop.Projects.Policies;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.GtkCore.GuiBuilder
 {
 	class GuiBuilderService
 	{
-		static string GuiBuilderLayout = "GUI Builder";
+		static string GuiBuilderLayout = "Visual Design";
 		
 #if DUMMY_STRINGS_FOR_TRANSLATION_DO_NOT_COMPILE
 		private void DoNotCompile ()
@@ -65,8 +61,6 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		static Stetic.Application steticApp;
 		
 		static bool generating;
-		static Stetic.CodeGenerationResult generationResult = null;
-		static Exception generatedException = null;
 		
 		static Stetic.IsolationMode IsolationMode = Stetic.IsolationMode.None;
 //		static Stetic.IsolationMode IsolationMode = Stetic.IsolationMode.ProcessUnix;
@@ -89,6 +83,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public static Stetic.Application SteticApp {
 			get {
+				// Stetic is not thread safe, so all has to be done in the gui thread
+				DispatchService.AssertGuiThread ();
 				if (steticApp == null) {
 					steticApp = Stetic.ApplicationFactory.CreateApplication (Stetic.IsolationMode.None);
 					steticApp.AllowInProcLibraries = false;
@@ -108,12 +104,12 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		static string OnMimeResolve (string url)
 		{
-			return MonoDevelop.Core.Gui.DesktopService.GetMimeTypeForUri (url);
+			return DesktopService.GetMimeTypeForUri (url);
 		}
 		
 		static void OnShowUrl (string url)
 		{
-			MonoDevelop.Core.Gui.DesktopService.ShowUrl (url);
+			DesktopService.ShowUrl (url);
 		}
 		
 		internal static void StoreConfiguration ()
@@ -175,7 +171,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		static void SetDesignerLayout ()
 		{
 			if (AutoSwitchGuiLayout && IdeApp.Workbench.CurrentLayout != GuiBuilderLayout) {
-				bool exists = Array.IndexOf (IdeApp.Workbench.Layouts, GuiBuilderLayout) != -1;
+				bool exists = IdeApp.Workbench.Layouts.Contains (GuiBuilderLayout);
 				defaultLayout = IdeApp.Workbench.CurrentLayout;
 				IdeApp.Workbench.CurrentLayout = GuiBuilderLayout;
 				if (!exists) {
@@ -406,8 +402,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			projects.Add (info.GuiBuilderProject.File);
 			
 			generating = true;
-			generationResult = null;
-			generatedException = null;
+			Stetic.CodeGenerationResult generationResult = null;
+			Exception generatedException = null;
 			
 			bool canGenerateInProcess = IsolationMode != Stetic.IsolationMode.None || info.GuiBuilderProject.SteticProject.CanGenerateCode;
 			
@@ -470,11 +466,11 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 					fname = Path.Combine (basePath, unit.Name) + ext;
 				StringWriter sw = new StringWriter ();
 				try {
+					foreach (CodeNamespace ns in unit.Namespaces)
+						ns.Comments.Add (new CodeCommentStatement ("This file has been generated by the GUI designer. Do not modify."));
 					provider.GenerateCodeFromCompileUnit (unit, sw, new CodeGeneratorOptions ());
-					// Remove the runtime version number from the file. It may generate unnecessary
-					// version control changes when upgrading the mono version.
 					string content = sw.ToString ();
-					content = content.Replace ("Mono Runtime Version: " + Environment.Version, "");
+					content = FormatGeneratedFile (fname, content, provider);
 					File.WriteAllText (fname, content);
 				} finally {
 					FileService.NotifyFileChanged (fname);
@@ -495,12 +491,10 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			ProjectFile pfile = prj.Files.GetFile (file);
 			if (pfile == null) {
-				string[] files = IdeApp.ProjectOperations.AddFilesToProject (prj, new string[] { file }, prj.BaseDirectory);
-				if (files.Length == 0)
+				var files = IdeApp.ProjectOperations.AddFilesToProject (prj, new string[] { file }, prj.BaseDirectory);
+				if (files.Count == 0 || files[0] == null)
 					return null;
-				if (files [0] == null)
-					return null;
-				pfile = prj.Files.GetFile (files[0]);
+				pfile = files [0];
 			}
 			if (pfile.BuildAction == BuildAction.EmbeddedResource) {
 				AlertButton embedButton = new AlertButton (GettextCatalog.GetString ("_Use as Source"));
@@ -513,6 +507,70 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			return pfile.FilePath;
 		}
 		
+		static string FormatGeneratedFile (string file, string content, CodeDomProvider provider)
+		{
+			content = StripHeaderAndBlankLines (content, provider);
+			string mt = DesktopService.GetMimeTypeForUri (file);
+			Formatter formatter = TextFileService.GetFormatter (mt);
+			if (formatter != null)
+				content = formatter.FormatText (PolicyService.InvariantPolicies, content);
+			return content;
+		}
+		
+		static string StripHeaderAndBlankLines (string text, CodeDomProvider provider)
+		{
+			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
+			doc.Text = text;
+			int realStartLine = 0;
+			for (int i = 0; i < doc.LineCount; i++) {
+				string lineText = doc.GetTextAt (doc.GetLine (i));
+				// Microsoft.NET generates "auto-generated" tags where Mono generates "autogenerated" tags.
+				if (lineText.Contains ("</autogenerated>") || lineText.Contains ("</auto-generated>")) {
+					realStartLine = i + 2;
+					break;
+				}
+			}
+			
+			// The Mono provider inserts additional blank lines, so strip them out
+			// But blank lines might actually be significant in other languages.
+			// We reformat the C# generated output to the user's coding style anyway, but the reformatter preserves blank lines
+			if (provider is Microsoft.CSharp.CSharpCodeProvider) {
+				bool previousWasBlank = false;
+				for (int i = 0; i < doc.LineCount; i++) {
+					Mono.TextEditor.LineSegment line = doc.GetLine (i);
+					bool isBlank, isBracket;
+					CheckLine (doc, line, out isBlank, out isBracket);
+					if (isBlank && previousWasBlank && line.Length > 0) {
+						((Mono.TextEditor.IBuffer)doc).Remove (line.Offset, line.Length);
+						i--;
+					}
+					previousWasBlank = isBlank || isBracket;
+				}
+			}
+			
+			int offset = doc.GetLine (realStartLine).Offset;
+			return doc.GetTextAt (offset, doc.Length - offset);
+		}
+
+		static void CheckLine (Mono.TextEditor.Document doc, Mono.TextEditor.LineSegment line, out bool isBlank, out bool isBracket)
+		{
+			isBlank = true;
+			isBracket = false;
+			for (int i = 0; i < line.Length; i++) {
+				char c = doc.GetCharAt (line.Offset + i);
+				if (c == '{') {
+					isBracket = true;
+					isBlank = false;
+				}
+				else if (!Char.IsWhiteSpace (c)) {
+					isBlank = false;
+					if (isBracket) {
+						isBracket = false;
+						break;
+					}
+				}
+			}
+		}
 	}
 
 
