@@ -29,6 +29,7 @@ using System.IO;
 using System.Collections.Generic;
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
+using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using System.Xml;
 using System.Text;
@@ -68,7 +69,7 @@ namespace MonoDevelop.Moonlight
 					var outFile = objDir.Combine (proj.LanguageBinding.GetFileName (pf.FilePath.FileName + ".g"));
 					buildData.Items.Add (new ProjectFile (outFile, BuildAction.Compile));
 					if (!File.Exists (outFile) || File.GetLastWriteTime (outFile) < File.GetLastWriteTime (pf.FilePath)) {
-						string rel = pf.RelativePath;
+						string rel = pf.ProjectVirtualPath;
 						monitor.Log.WriteLine ("Generating codebehind accessors for {0}...", rel);
 						BuildResult result = XamlG.GenerateFile (codeDomProvider, appName, pf.FilePath, rel, outFile);
 						if (result.Failed)
@@ -118,14 +119,9 @@ namespace MonoDevelop.Moonlight
 			}
 			
 			var si = new System.Diagnostics.ProcessStartInfo ();
-			foreach (KeyValuePair<string,string> env in runtime.GetToolsEnvironmentVariables (proj.TargetFramework)) {
-				if (env.Value == null) {
-					if (si.EnvironmentVariables.ContainsKey (env.Key))
-						si.EnvironmentVariables.Remove (env.Key);
-				} else {
-					si.EnvironmentVariables[env.Key] = env.Value;
-				}
-			}
+			var env = runtime.GetToolsExecutionEnvironment (proj.TargetFramework);
+			env.MergeTo (si);
+			
 			si.FileName = respack.EndsWith (".exe")? "mono" : respack;
 			si.WorkingDirectory = outfile.ParentDirectory;
 			
@@ -505,7 +501,7 @@ namespace MonoDevelop.Moonlight
 			foreach (ProjectFile pf in proj.Files) {
 				if (pf.BuildAction == BuildAction.Content) {
 					src.Add (pf.FilePath);
-					targ.Add (pf.RelativePath);
+					targ.Add (pf.ProjectVirtualPath);
 				}
 			}
 			
