@@ -25,30 +25,33 @@
 // THE SOFTWARE.
 
 using System;
-using System.Linq;
 using System.Collections.Generic;
-using System.IO;
 using System.Text;
 
 using ICSharpCode.NRefactory;
 using ICSharpCode.NRefactory.Ast;
-using ICSharpCode.NRefactory.PrettyPrinter;
 
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Content;
-using MonoDevelop.Ide.Gui.Dialogs;
-using MonoDevelop.Projects.CodeGeneration;
 using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
 using Mono.TextEditor;
 using MonoDevelop.Projects.Dom.Output;
+using MonoDevelop.Ide;
+using System.Linq;
 
 namespace MonoDevelop.Refactoring.ExtractMethod
 {
 	public class ExtractMethodRefactoring : RefactoringOperation
 	{
+		public override string AccelKey {
+			get {
+				var cmdInfo = IdeApp.CommandService.GetCommandInfo (RefactoryCommands.ExtractMethod);
+				if (cmdInfo != null && cmdInfo.AccelKey != null)
+					return cmdInfo.AccelKey.Replace ("dead_circumflex", "^");
+				return null;
+			}
+		}
+		
 		public ExtractMethodRefactoring ()
 		{
 			Name = "Extract Method";
@@ -56,8 +59,8 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 		
 		public override bool IsValid (RefactoringOptions options)
 		{
-			if (options.SelectedItem != null)
-				return false;
+//			if (options.SelectedItem != null)
+//				return false;
 			var buffer = options.Document.TextEditor;
 			if (buffer.SelectionStartPosition - buffer.SelectionEndPosition != 0) {
 				ParsedDocument doc = options.ParseDocument ();
@@ -82,11 +85,11 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 			ExtractMethodParameters param = CreateParameters (options);
 			if (param == null)
 				return;
-			if (Analyze (options, param, false) == null || param.VariablesToGenerate == null)
+			if (Analyze (options, param, false) == null || param.VariablesToGenerate == null) {
+				MessageService.ShowError (GettextCatalog.GetString ("Invalid selection for method extraction."));
 				return;
-			ExtractMethodDialog dialog = new ExtractMethodDialog (options, this, param);
-			dialog.TransientFor = IdeApp.Workbench.RootWindow;
-			dialog.Run ();
+			}
+			MessageService.ShowCustomDialog (new ExtractMethodDialog (options, this, param));
 		}
 		
 		public ExtractMethodParameters CreateParameters (RefactoringOptions options)
@@ -141,6 +144,11 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 				set;
 			}
 			
+			public InsertionPoint InsertionPoint {
+				get;
+				set;
+			}
+			
 			public DomLocation Location {
 				get;
 				set;
@@ -192,6 +200,56 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 			public bool OneChangedVariable { get; set; }
 		}
 		
+		static string GetIndent (string text)
+		{
+			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
+			doc.Text = text;
+			StringBuilder result = null;
+			for (int i = 1; i < doc.LineCount; i++) {
+				LineSegment line = doc.GetLine (i);
+				
+				StringBuilder lineIndent = new StringBuilder ();
+				foreach (char ch in doc.GetTextAt (line)) {
+					if (!char.IsWhiteSpace (ch))
+						break;
+					lineIndent.Append (ch);
+				}
+				if (line.EditableLength == lineIndent.Length)
+					continue;
+				if (result == null || lineIndent.Length < result.Length)
+					result = lineIndent;
+			}
+			if (result == null)
+				return "";
+			return result.ToString ();
+		}
+		
+		static string RemoveIndent (string text, string indent)
+		{
+			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
+			doc.Text = text;
+			StringBuilder result = new StringBuilder ();
+			foreach (LineSegment line in doc.Lines) {
+				string curLineIndent = line.GetIndentation (doc);
+				int offset = Math.Min (curLineIndent.Length, indent.Length);
+				result.Append (doc.GetTextBetween (line.Offset + offset, line.EndOffset));
+			}
+			return result.ToString ();
+		}
+		
+		static string AddIndent (string text, string indent)
+		{
+			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
+			doc.Text = text;
+			StringBuilder result = new StringBuilder ();
+			foreach (LineSegment line in doc.Lines) {
+				if (result.Length > 0)
+					result.Append (indent);
+				result.Append (doc.GetTextAt (line));
+			}
+			return result.ToString ();
+		}
+		
 		ICSharpCode.NRefactory.Ast.INode Analyze (RefactoringOptions options, ExtractMethodParameters param, bool fillParameter)
 		{
 			IResolver resolver = options.GetResolver ();
@@ -200,9 +258,23 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 				return null;
 
 			string text = options.Document.TextEditor.GetText (options.Document.TextEditor.SelectionStartPosition, options.Document.TextEditor.SelectionEndPosition);
-			param.Text = text;
+			
+			TextEditorData data = options.GetTextEditorData ();
+			var cu = provider.ParseFile (data.Document.GetTextAt (0, data.SelectionRange.Offset) + "MethodCall ();" + data.Document.GetTextAt (data.SelectionRange.EndOffset, data.Document.Length - data.SelectionRange.EndOffset));
+			
+			if (cu == null || provider.LastErrors.Count > 0) 
+				cu = provider.ParseFile (data.Document.GetTextAt (0, data.SelectionRange.Offset) + "MethodCall ()" + data.Document.GetTextAt (data.SelectionRange.EndOffset, data.Document.Length - data.SelectionRange.EndOffset));
+			
+			if (cu == null || provider.LastErrors.Count > 0) 
+				return null;
+			
+			param.Text = RemoveIndent (text, GetIndent (text)).TrimEnd ('\n', '\r');
+			
 			ICSharpCode.NRefactory.Ast.INode result = provider.ParseText (text);
-
+			if (cu == null || provider.LastErrors.Count > 0) {
+				return null;
+			}
+			
 			VariableLookupVisitor visitor = new VariableLookupVisitor (resolver, param.Location);
 			visitor.MemberLocation = new Location (param.DeclaringMember.Location.Column, param.DeclaringMember.Location.Line);
 			if (fillParameter) {
@@ -213,13 +285,25 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 					if (resolveResult != null)
 						param.ExpressionType = resolveResult.ResolvedType;
 				}
+				
+				var startLocation = data.Document.OffsetToLocation (data.SelectionRange.Offset);
+				var endLocation = data.Document.OffsetToLocation (data.SelectionRange.EndOffset);
+//				Console.WriteLine ("startLocation={0}, endLocation={1}", startLocation, endLocation);
+				
 				foreach (VariableDescriptor varDescr in visitor.VariableList.Where (v => !v.IsDefined && v.InitialValueUsed)) {
+					if (startLocation <= varDescr.Location && varDescr.Location < endLocation)
+						continue;
+//					Console.WriteLine (varDescr.Location);
+//					Console.WriteLine (startLocation <= varDescr.Location);
+//					Console.WriteLine (varDescr.Location < endLocation);
 					param.Parameters.Add (varDescr);
 				}
 				param.Variables = new List<VariableDescriptor> (visitor.Variables.Values);
 				foreach (VariableDescriptor varDescr in visitor.VariableList.Where (v => !v.IsDefined && param.Variables.Contains (v))) {
 					if (param.Parameters.Contains (varDescr))
 						continue;
+					if (startLocation <= varDescr.Location && varDescr.Location < endLocation)
+						continue;
 					param.Parameters.Add (varDescr);
 				}
 				
@@ -227,7 +311,6 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 				param.ChangedVariables = new HashSet<string> (visitor.Variables.Values.Where (v => v.GetsChanged).Select (v => v.Name));
 				
 				// analyze the variables outside of the selected text
-				TextEditorData data = options.GetTextEditorData ();
 				IMember member = param.DeclaringMember;
 			
 				int startOffset = data.Document.LocationToOffset (member.BodyRegion.Start.Line, member.BodyRegion.Start.Column);
@@ -237,11 +320,25 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 				text = data.Document.GetTextBetween (startOffset, data.SelectionRange.Offset) + data.Document.GetTextBetween (data.SelectionRange.EndOffset, endOffset);
 				ICSharpCode.NRefactory.Ast.INode parsedNode = provider.ParseText (text);
 				visitor = new VariableLookupVisitor (resolver, param.Location);
+				visitor.CutRegion = new DomRegion (data.MainSelection.MinLine, data.MainSelection.MaxLine);
 				visitor.MemberLocation = new Location (param.DeclaringMember.Location.Column, param.DeclaringMember.Location.Line);
 				if (parsedNode != null)
 					parsedNode.AcceptVisitor (visitor, null);
-				param.VariablesOutside = visitor.Variables;
-				param.OutsideVariableList = visitor.VariableList;
+				
+				
+				param.VariablesOutside = new Dictionary<string, VariableDescriptor> ();
+				foreach (var pair in visitor.Variables) {
+					if (startLocation < pair.Value.Location || endLocation >= pair.Value.Location) {
+						param.VariablesOutside.Add (pair.Key, pair.Value);
+					}
+				}
+				param.OutsideVariableList = new List<VariableDescriptor> ();
+				foreach (var v in visitor.VariableList) {
+					if (startLocation < v.Location || endLocation >= v.Location)
+						param.OutsideVariableList.Add (v);
+				}
+				
+				
 				
 				param.ChangedVariablesUsedOutside = new List<VariableDescriptor> (param.Variables.Where (v => v.GetsChanged && param.VariablesOutside.ContainsKey (v.Name)));
 				param.OneChangedVariable = result is BlockStatement;
@@ -297,9 +394,8 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 					invocation.Arguments.Add (new IdentifierExpression (var.Name));
 				}
 			}
-		//	string mimeType = DesktopService.GetMimeTypeForUri (options.Document.FileName);
+			//	string mimeType = DesktopService.GetMimeTypeForUri (options.Document.FileName);
 			TypeReference returnType = new TypeReference ("System.Void", true);
-
 			ICSharpCode.NRefactory.Ast.INode outputNode;
 			if (param.OneChangedVariable) {
 				string name = param.ChangedVariables.First ();
@@ -336,8 +432,9 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 			TextReplaceChange insertNewMethod = new TextReplaceChange ();
 			insertNewMethod.FileName = options.Document.FileName;
 			insertNewMethod.Description = string.Format (GettextCatalog.GetString ("Create new method {0} from selected statement(s)"), param.Name);
-			insertNewMethod.Offset = options.Document.TextEditor.GetPositionFromLineColumn (param.DeclaringMember.BodyRegion.End.Line, param.DeclaringMember.BodyRegion.End.Column);
-
+			insertNewMethod.RemovedChars = param.InsertionPoint.LineBefore == NewLineInsertion.Eol ? 0 : param.InsertionPoint.Location.Column;
+			insertNewMethod.Offset = data.Document.LocationToOffset (param.InsertionPoint.Location) - insertNewMethod.RemovedChars;
+			
 			ExtractMethodAstTransformer transformer = new ExtractMethodAstTransformer (param.VariablesToGenerate);
 			node.AcceptVisitor (transformer, null);
 			if (!param.OneChangedVariable && node is Expression) {
@@ -362,10 +459,10 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 				methodDecl.Body = new BlockStatement ();
 				methodDecl.Body.AddChild (new ReturnStatement (node as Expression));
 			}
-
+			
 			foreach (VariableDescriptor var in param.VariablesToDefine) {
 				BlockStatement block = methodDecl.Body;
-				LocalVariableDeclaration varDecl = new LocalVariableDeclaration (options.ShortenTypeName (var.ReturnType).ConvertToTypeReference());
+				LocalVariableDeclaration varDecl = new LocalVariableDeclaration (options.ShortenTypeName (var.ReturnType).ConvertToTypeReference ());
 				varDecl.Variables.Add (new VariableDeclaration (var.Name));
 				block.Children.Insert (0, varDecl);
 			}
@@ -391,9 +488,15 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 			
 			string indent = options.GetIndent (param.DeclaringMember);
 			StringBuilder methodText = new StringBuilder ();
-			methodText.AppendLine ();
-			methodText.Append (indent);
-			methodText.AppendLine ();
+			switch (param.InsertionPoint.LineBefore) {
+			case NewLineInsertion.Eol:
+				methodText.AppendLine ();
+				break;
+			case NewLineInsertion.BlankLine:
+				methodText.Append (indent);
+				methodText.AppendLine ();
+				break;
+			}
 			if (param.GenerateComment) {
 				methodText.Append (indent);
 				methodText.AppendLine ("/// <summary>");
@@ -403,11 +506,11 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 				methodText.AppendLine ("/// </summary>");
 				Ambience ambience = AmbienceService.GetAmbienceForFile (options.Document.FileName);
 				foreach (ParameterDeclarationExpression pde in methodDecl.Parameters) {
-						methodText.Append (indent);
+					methodText.Append (indent);
 					methodText.Append ("/// <param name=\"");
 					methodText.Append (pde.ParameterName);
 					methodText.Append ("\"> A ");
-					methodText.Append (ambience.GetString (pde.TypeReference.ConvertToReturnType (), OutputFlags.IncludeGenerics | OutputFlags.UseFullName) );
+					methodText.Append (ambience.GetString (pde.TypeReference.ConvertToReturnType (), OutputFlags.IncludeGenerics | OutputFlags.UseFullName));
 					methodText.Append (" </param>");
 					methodText.AppendLine ();
 				}
@@ -416,30 +519,39 @@ namespace MonoDevelop.Refactoring.ExtractMethod
 					methodText.AppendLine ("/// <returns>");
 					methodText.Append (indent);
 					methodText.Append ("/// A ");
-					methodText.AppendLine (ambience.GetString (methodDecl.TypeReference.ConvertToReturnType (), OutputFlags.IncludeGenerics | OutputFlags.UseFullName) );
+					methodText.AppendLine (ambience.GetString (methodDecl.TypeReference.ConvertToReturnType (), OutputFlags.IncludeGenerics | OutputFlags.UseFullName));
 					methodText.Append (indent);
 					methodText.AppendLine ("/// </returns>");
 				}
 			}
-	/*		foreach (VariableDescriptor var in variablesToOutput) {
-				TypeReference typeReference = options.ShortenTypeName (var.ReturnType).ConvertToTypeReference ();
-				ParameterDeclarationExpression pde = new ParameterDeclarationExpression (typeReference, var.Name);
-				if (!param.OneChangedVariable && param.ChangedVariables.Contains (var.Name))
-					pde.ParamModifier |= ICSharpCode.NRefactory.Ast.ParameterModifiers.Out;
-				methodDecl.Parameters.Add (pde);
-			}*/
 			
 			methodText.Append (indent);
+			
 			if (node is BlockStatement) {
 				string text = provider.OutputNode (options.Dom, methodDecl, indent).Trim ();
 				int emptyStatementMarker = text.LastIndexOf (';');
 				if (param.OneChangedVariable)
 					emptyStatementMarker = text.LastIndexOf (';', emptyStatementMarker - 1);
-				text = text.Substring (0, emptyStatementMarker) + param.Text + text.Substring (emptyStatementMarker + 1);
-				methodText.Append (text);
+				StringBuilder sb = new StringBuilder ();
+				sb.Append (text.Substring (0, emptyStatementMarker));
+				sb.Append (AddIndent (param.Text, indent + "\t"));
+				sb.Append (text.Substring (emptyStatementMarker + 1));
+				
+				methodText.Append (sb.ToString ());
 			} else {
 				methodText.Append (provider.OutputNode (options.Dom, methodDecl, options.GetIndent (param.DeclaringMember)).Trim ());
 			}
+			
+			switch (param.InsertionPoint.LineAfter) {
+			case NewLineInsertion.Eol:
+				methodText.AppendLine ();
+				break;
+			case NewLineInsertion.BlankLine:
+				methodText.AppendLine ();
+				methodText.AppendLine ();
+				methodText.Append (indent);
+				break;
+			}
 			insertNewMethod.InsertedText = methodText.ToString ();
 			result.Add (insertNewMethod);
 
