@@ -24,8 +24,7 @@
 // THE SOFTWARE.
 
 using System;
-using MonoDevelop.Core.Gui.Dialogs;
-using MonoDevelop.Core.Gui;
+using MonoDevelop.Ide.Gui.Dialogs; 
 
 namespace MonoDevelop.SourceEditor.OptionPanels
 {
