@@ -27,17 +27,13 @@
 //
 
 using System;
-using System.Collections;
 
-using MonoDevelop.Projects;
 using MonoDevelop.Core;
-using MonoDevelop.Ide.Gui.Pads;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.NUnit.Commands;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Commands;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui.Components;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.NUnit
 {
@@ -164,6 +160,17 @@ namespace MonoDevelop.NUnit
 	{
 		[CommandHandler (TestCommands.ShowTestCode)]
 		protected void OnShowTest ()
+		{
+			UnitTest test = CurrentNode.DataItem as UnitTest;
+			SourceCodeLocation loc = null;
+			UnitTestResult res = test.GetLastResult ();
+			loc = test.SourceCodeLocation;
+			if (loc != null)
+				IdeApp.Workbench.OpenDocument (loc.FileName, loc.Line, loc.Column, true);
+		}
+		
+		[CommandHandler (TestCommands.GoToFailure)]
+		protected void OnShowFailure ()
 		{
 			UnitTest test = CurrentNode.DataItem as UnitTest;
 			SourceCodeLocation loc = null;
