@@ -27,20 +27,17 @@
 //
 
 using System;
-using System.IO;
 using System.Collections;
 using System.Collections.Generic;
 using System.Threading;
 
 using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
-using MonoDevelop.Core.ProgressMonitoring;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Core.Gui.Dialogs;
 using Mono.Addins;
 using MonoDevelop.Projects;
 using NUnit.Core;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.NUnit
 {
@@ -57,6 +54,8 @@ namespace MonoDevelop.NUnit
 			IdeApp.Workspace.ReferenceRemovedFromProject += OnWorkspaceChanged;
 			IdeApp.Workspace.WorkspaceItemOpened += OnWorkspaceChanged;
 			IdeApp.Workspace.WorkspaceItemClosed += OnWorkspaceChanged;
+			IdeApp.Workspace.ItemAddedToSolution += OnWorkspaceChanged;
+			IdeApp.Workspace.ItemRemovedFromSolution += OnWorkspaceChanged;
 			
 			ProjectService ps = MonoDevelop.Projects.Services.ProjectService;
 			ps.DataContext.IncludeType (typeof(UnitTestOptionsSet));
@@ -253,9 +252,7 @@ namespace MonoDevelop.NUnit
 		{
 			Properties properties = new Properties ();
 			properties.Set ("UnitTest", test);
-			UnitTestOptionsDialog optionsDialog = new UnitTestOptionsDialog (IdeApp.Workbench.RootWindow, properties);
-			optionsDialog.TransientFor = IdeApp.Workbench.RootWindow;
-			optionsDialog.Run ();
+			MessageService.ShowCustomDialog (new UnitTestOptionsDialog (IdeApp.Workbench.RootWindow, properties));
 		}
 		
 		void NotifyTestSuiteChanged ()
