@@ -1,22 +1,28 @@
 //  VBLanguageBinding.cs
 //
-//  This file was derived from a file from #Develop. 
+//  This file was derived from a file from #Develop, and relicensed
+//  by Markus Palme to MIT/X11
 //
 //  Copyright (C) 2001-2007 Markus Palme <MarkusPalme@gmx.de>
 // 
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
 // 
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 using System;
 using System.IO;
@@ -48,26 +54,31 @@ namespace MonoDevelop.VBNetBinding
 			}
 		}
 		
+		public string ProjectStockIcon {
+			get { 
+				return "md-project";
+			}
+		}
+		
 		public bool IsSourceCodeFile (string fileName)
 		{
 			return Path.GetExtension(fileName) == ".vb";
 		}
 		
-		public BuildResult Compile (ProjectItemCollection items, DotNetProjectConfiguration configuration, IProgressMonitor monitor)
+		public BuildResult Compile (ProjectItemCollection items, DotNetProjectConfiguration configuration, ConfigurationSelector configSelector, IProgressMonitor monitor)
 		{
 			Debug.Assert(compilerServices != null);
-			return compilerServices.Compile (items, configuration, monitor);
+			return compilerServices.Compile (items, configuration, configSelector, monitor);
 		}
 		
 		public ConfigurationParameters CreateCompilationParameters (XmlElement projectOptions)
 		{
 			return new VBCompilerParameters ();
 		}
-
-		public string CommentTag
-		{
-			get { return "'"; }
-		}
+		
+		public string SingleLineCommentTag { get { return "'"; } }
+		public string BlockCommentStartTag { get { return null; } }
+		public string BlockCommentEndTag { get { return null; } }
 		
 		public CodeDomProvider GetCodeDomProvider ()
 		{
