@@ -1,10 +1,8 @@
 
 using System;
-using System.Collections;
 using Gtk;
 using Gdk;
-using MonoDevelop.Core;
-using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.VersionControl.Views
 {
@@ -14,11 +12,14 @@ namespace MonoDevelop.VersionControl.Views
 		Pango.FontDescription font;
 		bool diffMode;
 		int width, height, lineHeight;
-		string[] lines;
+		string[] lines;		
+		int selectedLine = -1;
+		TreePath selctedPath;
+		TreePath path;
 
 		public CellRendererDiff()
 		{
-			font = Pango.FontDescription.FromString (IdeApp.Services.PlatformService.DefaultMonospaceFont);
+			font = Pango.FontDescription.FromString (DesktopService.DefaultMonospaceFont);
 		}
 		
 		void DisposeLayout ()
@@ -44,16 +45,17 @@ namespace MonoDevelop.VersionControl.Views
 		public void Reset ()
 		{
 		}
-		
-		public void InitCell (Widget container, bool diffMode, string text, string path)
+
+		public void InitCell (Widget container, bool diffMode, string[] lines, TreePath path)
 		{
 			if (isDisposed)
 				return;
+			this.lines = lines;
 			this.diffMode = diffMode;
+			this.path = path;
 			
 			if (diffMode) {
-				if (text.Length > 0) {
-					lines = text.Split ('\n');
+				if (lines != null && lines.Length > 0) {
 					int maxlen = -1;
 					int maxlin = -1;
 					for (int n=0; n<lines.Length; n++) {
@@ -72,7 +74,7 @@ namespace MonoDevelop.VersionControl.Views
 			}
 			else {
 				DisposeLayout ();
-				layout = CreateLayout (container, text);
+				layout = CreateLayout (container, string.Join (Environment.NewLine, lines));
 				layout.GetPixelSize (out width, out height);
 			}
 		}
@@ -95,9 +97,18 @@ namespace MonoDevelop.VersionControl.Views
 			if (isDisposed)
 				return;
 			if (diffMode) {
+				
+				if (path.Equals (selctedPath)) {
+					selectedLine = -1;
+					selctedPath = null;
+				}
+				
 				int w, maxy;
 				window.GetSize (out w, out maxy);
 				
+				var treeview = widget as FileTreeView;
+				var p = treeview != null? treeview.CursorLocation : null;
+				
 				int recty = cell_area.Y;
 				int recth = cell_area.Height - 1;
 				if (recty < 0) {
@@ -121,22 +132,43 @@ namespace MonoDevelop.VersionControl.Views
 				infoGC.RgbFgColor = new Color (0xa5, 0x2a, 0x2a);
 				
 				int y = cell_area.Y + 2;
+				int cline = 1;
+				bool inHeader = true;
 				
 				for (int n=0; n<lines.Length; n++, y += lineHeight) {
-					if (y + lineHeight < 0)
-						continue;
-					if (y > maxy)
-						break;
+					
 					string line = lines [n];
 					if (line.Length == 0)
 						continue;
 					
+					char tag = line [0];
+
+					// Keep track of the real file line
+					int thisLine = cline;
+					if (tag == '@') {
+						int l = ParseCurrentLine (line);
+						if (l != -1) cline = thisLine = l;
+						inHeader = false;
+					} else if (tag != '-' && !inHeader)
+						cline++;
+					
+					if (y + lineHeight < 0)
+						continue;
+					if (y > maxy)
+						break;
+					
 					Gdk.GC gc;
-					switch (line[0]) {
-					case '-': gc = removedGC; break;
-					case '+': gc = addedGC; break;
-					case '@': gc = infoGC; break;
-					default: gc = normalGC; break;
+					switch (tag) {
+						case '-': gc = removedGC; break;
+						case '+': gc = addedGC; break;
+						case '@': gc = infoGC; break;
+						default: gc = normalGC; break;
+					}
+
+					if (p.HasValue && p.Value.X >= cell_area.X && p.Value.X <= cell_area.Right && p.Value.Y >= y && p.Value.Y < y + lineHeight) {
+						window.DrawRectangle (widget.Style.BaseGC (Gtk.StateType.Prelight), true, cell_area.X, y, cell_area.Width - 1, lineHeight);
+						selectedLine = thisLine;
+						selctedPath = path;
 					}
 					
 					layout.SetText (line);
@@ -172,7 +204,26 @@ namespace MonoDevelop.VersionControl.Views
 			else
 				return StateType.Normal;
 		}
-	
 		
-}
+		int ParseCurrentLine (string line)
+		{
+			int i = line.IndexOf ('+');
+			if (i == -1) return -1;
+			i++;
+			int j = line.IndexOf (',', i);
+			if (j == -1) return -1;
+			int cline;
+			if (!int.TryParse (line.Substring (i, j - i), out cline))
+			    return -1;
+			return cline;
+		}
+		
+		public int GetSelectedLine (TreePath cpath)
+		{
+			if (cpath.Equals (selctedPath))
+				return selectedLine;
+			else
+				return -1;
+		}
+	}
 }
