@@ -40,11 +40,13 @@ namespace Mono.Debugging.Client
 	public delegate void ThreadEventHandler(int thread_id);
 	public delegate bool ExceptionHandler (Exception ex);
 	public delegate string TypeResolverHandler (string identifier, SourceLocation location);
+	public delegate void BreakpointTraceHandler (BreakEvent be, string trace);
+	public delegate IExpressionEvaluator GetExpressionEvaluatorHandler (string extension);
 	
 	public abstract class DebuggerSession: IDisposable
 	{
 		InternalDebuggerSession frontend;
-		Dictionary<BreakEvent,object> breakpoints = new Dictionary<BreakEvent,object> ();
+		Dictionary<BreakEvent,BreakEventInfo> breakpoints = new Dictionary<BreakEvent,BreakEventInfo> ();
 		bool isRunning;
 		bool started;
 		BreakpointStore breakpointStore;
@@ -60,6 +62,18 @@ namespace Mono.Debugging.Client
 		DebuggerSessionOptions options;
 		Dictionary<string,string> resolvedExpressionCache = new Dictionary<string, string> ();
 		
+		struct BreakEventInfo {
+			// Handle is the native debugger breakpoint handle
+			public object Handle;
+			// IsValid is always true unless the subclass explicitly sets it to false using SetBreakEventStatus.
+			public bool IsValid;
+			
+			public BreakEventInfo (object handle) {
+				Handle = handle;
+				IsValid = true;
+			}
+		}
+		
 		ProcessInfo[] currentProcesses;
 		
 		public event EventHandler<TargetEventArgs> TargetEvent;
@@ -87,12 +101,12 @@ namespace Mono.Debugging.Client
 		
 		public virtual void Dispose ()
 		{
-			lock (slock) {
+			Dispatch (delegate {
 				if (!disposed) {
 					disposed = true;
 					Breakpoints = null;
 				}
-			}
+			});
 		}
 		
 		public ExceptionHandler ExceptionHandler {
@@ -100,8 +114,12 @@ namespace Mono.Debugging.Client
 			set { exceptionHandler = value; }
 		}
 		
+		public BreakpointTraceHandler BreakpointTraceHandler { get; set; }
+		
 		public TypeResolverHandler TypeResolverHandler { get; set; }
 
+		public GetExpressionEvaluatorHandler GetExpressionEvaluator { get; set; }		
+		
 		public BreakpointStore Breakpoints {
 			get {
 				lock (slock) {
@@ -114,8 +132,10 @@ namespace Mono.Debugging.Client
 				lock (slock) {
 					if (breakpointStore != null) {
 						if (started) {
-							foreach (BreakEvent bp in breakpointStore)
+							foreach (BreakEvent bp in breakpointStore) {
 								RemoveBreakEvent (bp);
+								Breakpoints.NotifyStatusChanged (bp);
+							}
 						}
 						breakpointStore.BreakEventAdded -= OnBreakpointAdded;
 						breakpointStore.BreakEventRemoved -= OnBreakpointRemoved;
@@ -329,14 +349,48 @@ namespace Mono.Debugging.Client
 			}
 		}
 		
+		/// <summary>
+		/// Returns true if the breakpoint is valid for this debugger session.
+		/// It may be invalid, for example, if the breakpoint was placed in a line
+		/// that has no code.
+		/// </summary>
 		public bool IsBreakEventValid (BreakEvent be)
 		{
 			if (!started)
 				return true;
 			
-			object handle;
+			BreakEventInfo binfo;
 			lock (breakpoints) {
-				return (breakpoints.TryGetValue (be, out handle) && handle != null);
+				return (breakpoints.TryGetValue (be, out binfo) && binfo.IsValid && binfo.Handle != null);
+			}
+		}
+		
+		/// <summary>
+		/// This method can be used by subclasses to set the validity of a breakpoint.
+		/// </summary>
+		protected void SetBreakEventStatus (BreakEvent be, bool isValid)
+		{
+			lock (breakpoints) {
+				BreakEventInfo bi;
+				if (!breakpoints.TryGetValue (be, out bi))
+					bi = new BreakEventInfo (null);
+				if (bi.IsValid != isValid) {
+					bi.IsValid = isValid;
+					breakpoints [be] = bi;
+					Breakpoints.NotifyStatusChanged (be);
+				}
+			}
+		}
+
+		void SetBreakEventHandle (BreakEvent be, object handle)
+		{
+			lock (breakpoints) {
+				BreakEventInfo bi;
+				if (!breakpoints.TryGetValue (be, out bi))
+					bi = new BreakEventInfo (handle);
+				else
+					bi.Handle = handle;
+				breakpoints [be] = bi;
 			}
 		}
 
@@ -357,7 +411,7 @@ namespace Mono.Debugging.Client
 			}
 
 			lock (breakpoints) {
-				breakpoints.Add (be, handle);
+				SetBreakEventHandle (be, handle);
 				Breakpoints.NotifyStatusChanged (be);
 			}
 		}
@@ -370,7 +424,6 @@ namespace Mono.Debugging.Client
 					try {
 						if (handle != null)
 							OnRemoveBreakEvent (handle);
-						breakpoints.Remove (be);
 					} catch (Exception ex) {
 						if (started)
 							OnDebuggerOutput (false, ex.Message);
@@ -378,6 +431,7 @@ namespace Mono.Debugging.Client
 						return false;
 					}
 				}
+				breakpoints.Remove (be);
 				return true;
 			}
 		}
@@ -407,7 +461,7 @@ namespace Mono.Debugging.Client
 						object newHandle = OnUpdateBreakEvent (handle, be);
 						if (newHandle != handle && (newHandle == null || !newHandle.Equals (handle))) {
 							// Update the handle if it has changed, and notify the status change
-							breakpoints [be] = newHandle;
+							SetBreakEventHandle (be, newHandle);
 						}
 						Breakpoints.NotifyStatusChanged (be);
 					} else {
@@ -416,7 +470,7 @@ namespace Mono.Debugging.Client
 							handle = OnInsertBreakEvent (be, be.Enabled);
 							if (handle != null) {
 								// This time worked
-								breakpoints [be] = handle;
+								SetBreakEventHandle (be, handle);
 								Breakpoints.NotifyStatusChanged (be);
 							}
 						} catch (Exception ex) {
@@ -482,7 +536,13 @@ namespace Mono.Debugging.Client
 		
 		protected bool GetBreakpointHandle (BreakEvent be, out object handle)
 		{
-			return breakpoints.TryGetValue (be, out handle);
+			BreakEventInfo binfo;
+			if (!breakpoints.TryGetValue (be, out binfo)) {
+				handle = null;
+				return false;
+			}
+			handle = binfo.Handle;
+			return true;
 		}
 		
 		public DebuggerSessionOptions Options {
@@ -491,6 +551,7 @@ namespace Mono.Debugging.Client
 
 		public EvaluationOptions EvaluationOptions {
 			get { return options.EvaluationOptions; }
+			set { options.EvaluationOptions = value; }
 		}
 
 		public void Continue ()
@@ -589,14 +650,49 @@ namespace Mono.Debugging.Client
 				string key = expression + " " + location;
 				string resolved;
 				if (!resolvedExpressionCache.TryGetValue (key, out resolved)) {
-					resolved = OnResolveExpression (expression, location);
+					try {
+						resolved = OnResolveExpression (expression, location);
+					} catch (Exception ex) {
+						OnDebuggerOutput (true, "Error while resolving expression: " + ex.Message);
+					}
 					resolvedExpressionCache [key] = resolved;
 				}
-				return resolved;
+				return resolved ?? expression;
 			}
 		}
 		
-		Mono.Debugging.Evaluation.NRefactoryEvaluator defaultResolver = new Mono.Debugging.Evaluation.NRefactoryEvaluator ();
+		Mono.Debugging.Evaluation.ExpressionEvaluator defaultResolver = new Mono.Debugging.Evaluation.NRefactoryEvaluator ();
+		Dictionary <string, IExpressionEvaluator> evaluators = new Dictionary <string, IExpressionEvaluator> ();
+
+		public IExpressionEvaluator FindExpressionEvaluator (StackFrame frame)
+		{
+			if (GetExpressionEvaluator == null)
+				return null;
+
+			string fn = frame.SourceLocation == null ? null : frame.SourceLocation.Filename;
+			if (String.IsNullOrEmpty (fn))
+				return null;
+
+			fn = System.IO.Path.GetExtension (fn);
+			IExpressionEvaluator result;
+			if (evaluators.TryGetValue (fn, out result))
+				return result;
+
+			result = GetExpressionEvaluator(fn);
+
+			evaluators[fn] = result;
+
+			return result;
+		}
+
+		public Mono.Debugging.Evaluation.ExpressionEvaluator GetResolver (StackFrame frame)
+		{
+			IExpressionEvaluator result = FindExpressionEvaluator (frame);
+			if (result == null)
+				return defaultResolver;
+			return result.Evaluator;
+		}
+		
 		
 		protected virtual string OnResolveExpression (string expression, SourceLocation location)
 		{
@@ -756,10 +852,10 @@ namespace Mono.Debugging.Client
 		{
 			lock (breakpoints) {
 				// Make a copy of the breakpoints table since it can be modified while iterating
-				Dictionary<BreakEvent, object> breakpointsCopy = new Dictionary<BreakEvent, object> (breakpoints);
-				foreach (KeyValuePair<BreakEvent, object> bps in breakpointsCopy) {
+				Dictionary<BreakEvent, BreakEventInfo> breakpointsCopy = new Dictionary<BreakEvent, BreakEventInfo> (breakpoints);
+				foreach (KeyValuePair<BreakEvent, BreakEventInfo> bps in breakpointsCopy) {
 					Breakpoint bp = bps.Key as Breakpoint;
-					if (bp != null && bps.Value == null) {
+					if (bp != null && bps.Value.Handle == null) {
 						if (string.Compare (System.IO.Path.GetFullPath (bp.FileName), fullFilePath, System.IO.Path.DirectorySeparatorChar == '\\') == 0)
 							UpdateBreakEvent (bp);
 					}
@@ -772,25 +868,25 @@ namespace Mono.Debugging.Client
 			List<BreakEvent> toUpdate = new List<BreakEvent> ();
 			lock (breakpoints) {
 				// Make a copy of the breakpoints table since it can be modified while iterating
-				Dictionary<BreakEvent, object> breakpointsCopy = new Dictionary<BreakEvent, object> (breakpoints);
-				foreach (KeyValuePair<BreakEvent, object> bps in breakpointsCopy) {
+				Dictionary<BreakEvent, BreakEventInfo> breakpointsCopy = new Dictionary<BreakEvent, BreakEventInfo> (breakpoints);
+				foreach (KeyValuePair<BreakEvent, BreakEventInfo> bps in breakpointsCopy) {
 					Breakpoint bp = bps.Key as Breakpoint;
-					if (bp != null && bps.Value != null) {
+					if (bp != null && bps.Value.Handle != null) {
 						if (System.IO.Path.GetFullPath (bp.FileName) == fullFilePath)
 							toUpdate.Add (bp);
 					}
 				}
 				foreach (BreakEvent be in toUpdate) {
-					breakpoints[be] = null;
+					SetBreakEventHandle (be, null);
 					Breakpoints.NotifyStatusChanged (be);
 				}
 			}
 		}
-
+		
 		BreakEvent GetBreakEvent (object handle)
 		{
-			foreach (KeyValuePair<BreakEvent,object> e in breakpoints) {
-				if (handle == e.Value || handle.Equals (e.Value))
+			foreach (KeyValuePair<BreakEvent,BreakEventInfo> e in breakpoints) {
+				if (handle == e.Value.Handle || handle.Equals (e.Value.Handle))
 					return e.Key;
 			}
 			return null;
@@ -825,6 +921,12 @@ namespace Mono.Debugging.Client
 			if (ev != null) {
 				ev.LastTraceValue = value;
 				ev.NotifyUpdate ();
+				if (value != null) {
+					if (BreakpointTraceHandler != null)
+						BreakpointTraceHandler (ev, value);
+					else
+						OnDebuggerOutput (false, value + "\n");
+				}
 			}
 		}
 		
