@@ -30,11 +30,11 @@ using System;
 using System.Text;
 using System.Text.RegularExpressions;
 using System.Collections.Generic;
+using System.Linq;
 
 namespace Mono.TextEditor.Vi
 {
-	
-	
+
 	public class ViEditMode : EditMode
 	{
 		bool searchBackward;
@@ -43,8 +43,30 @@ namespace Mono.TextEditor.Vi
 		State state;
 		const string substMatch = @"^:s(?<sep>.)(?<pattern>.+?)\k<sep>(?<replacement>.*?)(\k<sep>(?<trailer>i?))?$";
 		StringBuilder commandBuffer = new StringBuilder ();
+		Dictionary<char,ViMark> marks = new Dictionary<char, ViMark>();
+		Dictionary<char,ViMacro> macros = new Dictionary<char, ViMacro>();
+		string statusText = "";
 		
-		public virtual string Status { get; protected set; }
+		/// <summary>
+		/// The macro currently being implemented. Will be set to null and checked as a flag when required.
+		/// </summary>
+		ViMacro currentMacro;
+		
+		public virtual string Status {
+		
+			get {
+				return statusText;
+			}
+			
+			protected set {
+				if (currentMacro == null) {
+					statusText = value;
+				} else {
+					statusText = value + " recording";
+				}
+			}
+		
+		}
 		
 		protected virtual string RunExCommand (string command)
 		{
@@ -91,21 +113,41 @@ namespace Mono.TextEditor.Vi
 					break;	
 				}
 				break;
-
+				
 			case '?':
 			case '/':
 				searchBackward = ('?' == command[0]);
 				if (1 < command.Length) {
 					Editor.HighlightSearchPattern = true;
 					Editor.SearchEngine = new RegexSearchEngine ();
-					Editor.SearchPattern = command.Substring (1);
+					var pattern = command.Substring (1);
+					Editor.SearchPattern = pattern;
+					var caseSensitive = pattern.ToCharArray ().Any (c => char.IsUpper (c));
+					Editor.SearchEngine.SearchRequest.CaseSensitive = caseSensitive;
 				}
 				return Search ();
 			}
-
+			
 			return "Command not recognised";
 		}
 		
+		void SearchWordAtCaret ()
+		{
+			Editor.SearchEngine = new RegexSearchEngine ();
+			var s = Data.FindCurrentWordStart (Data.Caret.Offset);
+			var e = Data.FindCurrentWordEnd (Data.Caret.Offset);
+			if (s < 0 || e <= s)
+				return;
+			
+			var word = Document.GetTextBetween (s, e);
+			//use negative lookahead and lookbehind for word characters to make sure we only get fully matching words
+			word = "(?<!\\w)" + System.Text.RegularExpressions.Regex.Escape (word) + "(?!\\w)";
+			Editor.SearchPattern = word;
+			Editor.SearchEngine.SearchRequest.CaseSensitive = true;
+			searchBackward = false;
+			Search ();
+		}
+		
 		public override bool WantsToPreemptIM {
 			get {
 				return state != State.Insert && state != State.Replace;
@@ -122,6 +164,16 @@ namespace Mono.TextEditor.Vi
 			}
 		}
 		
+		protected override void CaretPositionChanged ()
+		{
+			if (state == State.Replace || state == State.Insert || state == State.Visual)
+				return;
+			else if (state == ViEditMode.State.Normal || state == ViEditMode.State.Unknown)
+				ViActions.RetreatFromLineEnd (Data);
+			else
+				Reset ("");
+		}
+		
 		void CheckVisualMode ()
 		{
 			if (state == ViEditMode.State.Visual || state == ViEditMode.State.Visual) {
@@ -140,14 +192,35 @@ namespace Mono.TextEditor.Vi
 			if (data == null)
 				return;
 			data.ClearSelection ();
-			if (CaretMode.Block != data.Caret.Mode)
+			
+			//Editor can be null during GUI-less tests
+			if (Editor != null)
+				Editor.HighlightSearchPattern = false;
+			
+			if (CaretMode.Block != data.Caret.Mode) {
 				data.Caret.Mode = CaretMode.Block;
+				if (data.Caret.Column > 0)
+					data.Caret.Column--;
+			}
+			ViActions.RetreatFromLineEnd (data);
+		}
+		
+		protected override void OnAddedToEditor (TextEditorData data)
+		{
+			data.Caret.Mode = CaretMode.Block;
+			ViActions.RetreatFromLineEnd (data);
+		}
+		
+		protected override void OnRemovedFromEditor (TextEditorData data)
+		{
+			data.Caret.Mode = CaretMode.Insert;
 		}
 		
 		void Reset (string status)
 		{
-			ResetEditorState (Data);
 			state = State.Normal;
+			ResetEditorState (Data);
+			
 			commandBuffer.Length = 0;
 			Status = status;
 		}
@@ -157,17 +230,40 @@ namespace Mono.TextEditor.Vi
 			return ViActionMaps.GetInsertKeyAction (key, modifier) ??
 				ViActionMaps.GetDirectionKeyAction (key, modifier);
 		}
-		
+
 		protected override void HandleKeypress (Gdk.Key key, uint unicodeKey, Gdk.ModifierType modifier)
 		{
+		
 			// Reset on Esc, Ctrl-C, Ctrl-[
-			if (key == Gdk.Key.Escape || 
-			    ((key == Gdk.Key.c || key == Gdk.Key.bracketleft) && (modifier & Gdk.ModifierType.ControlMask) != 0)) {
-				Reset (string.Empty);
+			if (key == Gdk.Key.Escape) {
+				if (currentMacro != null) {
+					// Record Escapes into the macro since it actually does something
+					ViMacro.KeySet toAdd = new ViMacro.KeySet();
+					toAdd.Key = key;
+					toAdd.Modifiers = modifier;
+					toAdd.UnicodeKey = unicodeKey;
+					currentMacro.KeysPressed.Enqueue(toAdd);
+				}
+				Reset(string.Empty);
 				return;
+			} else if (((key == Gdk.Key.c || key == Gdk.Key.bracketleft) && (modifier & Gdk.ModifierType.ControlMask) != 0)) {
+				Reset (string.Empty);
+				if (currentMacro != null) {
+					// Otherwise remove the macro from the pool
+					macros.Remove(currentMacro.MacroCharacter);
+					currentMacro = null;
+				}
+				return;
+			} else if (currentMacro != null && !((char)unicodeKey == 'q' && modifier == Gdk.ModifierType.None)) {
+				ViMacro.KeySet toAdd = new ViMacro.KeySet();
+				toAdd.Key = key;
+				toAdd.Modifiers = modifier;
+				toAdd.UnicodeKey = unicodeKey;
+				currentMacro.KeysPressed.Enqueue(toAdd);
 			}
 			
 			Action<TextEditorData> action = null;
+			bool lineAction = false;
 			
 			switch (state) {
 			case State.Unknown:
@@ -195,7 +291,8 @@ namespace Mono.TextEditor.Vi
 						goto case 'i';
 					
 					case 'a':
-						RunAction (ViActions.Right);
+						//use CaretMoveActions so that we can move past last character on line end
+						RunAction (CaretMoveActions.Right);
 						goto case 'i';
 					case 'i':
 						Caret.Mode = CaretMode.Insert;
@@ -217,10 +314,8 @@ namespace Mono.TextEditor.Vi
 						
 					case 'v':
 						Status = "-- VISUAL --";
-						if (Data.MainSelection == null && Caret.Offset+1 < Data.Document.Length)
-							Data.SetSelection (Caret.Offset, Caret.Offset+1);
-						Data.MainSelection.Anchor = Caret.Location;
 						state = State.Visual;
+						RunAction (ViActions.VisualSelectionFromMoveAction (ViActions.Right));
 						return;
 						
 					case 'd':
@@ -259,14 +354,19 @@ namespace Mono.TextEditor.Vi
 						return;
 						
 					case 'x':
+						if (Data.Caret.Column == Data.Document.GetLine (Data.Caret.Line).EditableLength)
+							return;
 						Status = string.Empty;
 						if (!Data.IsSomethingSelected)
 							RunActions (SelectionActions.FromMoveAction (CaretMoveActions.Right), ClipboardActions.Cut);
 						else
 							RunAction (ClipboardActions.Cut);
+						ViActions.RetreatFromLineEnd (Data);
 						return;
 						
 					case 'X':
+						if (Data.Caret.Column == 0)
+							return;
 						Status = string.Empty;
 						if (!Data.IsSomethingSelected && 0 < Caret.Offset)
 							RunActions (SelectionActions.FromMoveAction (CaretMoveActions.Left), ClipboardActions.Cut);
@@ -349,9 +449,36 @@ namespace Mono.TextEditor.Vi
 						Status = "z";
 						state = State.Fold;
 						return;
+						
+					case 'm':
+						Status = "m";
+						state = State.Mark;
+						return;
+						
+					case '`':
+						Status = "`";
+						state = State.GoToMark;
+						return;
+						
+					case '@':
+						Status = "@";
+						state = State.PlayMacro;
+						return;
+	
+					case 'q':
+						if (currentMacro == null) {
+							Status = "q";
+							state = State.NameMacro;
+							return;
+						} 
+						currentMacro = null;
+						Reset("Macro Recorded");
+						return;
+					case '*':
+						SearchWordAtCaret ();
+						return;
 					}
 					
-					
 				}
 				
 				action = ViActionMaps.GetNavCharAction ((char)unicodeKey);
@@ -369,9 +496,10 @@ namespace Mono.TextEditor.Vi
 				
 			case State.Delete:
 				if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
-				     && key == Gdk.Key.d))
+				     && unicodeKey == 'd'))
 				{
 					action = SelectionActions.LineActionFromMoveAction (CaretMoveActions.LineEnd);
+					lineAction = true;
 				} else {
 					action = ViActionMaps.GetNavCharAction ((char)unicodeKey);
 					if (action == null)
@@ -381,7 +509,10 @@ namespace Mono.TextEditor.Vi
 				}
 				
 				if (action != null) {
-					RunActions (action, ClipboardActions.Cut);
+					if (lineAction)
+						RunActions (action, ClipboardActions.Cut, CaretMoveActions.LineFirstNonWhitespace);
+					else
+						RunActions (action, ClipboardActions.Cut);
 					Reset ("");
 				} else {
 					Reset ("Unrecognised motion");
@@ -391,13 +522,12 @@ namespace Mono.TextEditor.Vi
 
 			case State.Yank:
 				int offset = Caret.Offset;
-				bool lineyank = false;
 				
 				if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
-				     && key == Gdk.Key.y))
+				     && unicodeKey == 'y'))
 				{
 					action = SelectionActions.LineActionFromMoveAction (CaretMoveActions.LineEnd);
-					lineyank = true;
+					lineAction	= true;
 				} else {
 					action = ViActionMaps.GetNavCharAction ((char)unicodeKey);
 					if (action == null)
@@ -408,7 +538,7 @@ namespace Mono.TextEditor.Vi
 				
 				if (action != null) {
 					RunAction (action);
-					if (Data.IsSomethingSelected && !lineyank)
+					if (Data.IsSomethingSelected && !lineAction)
 						offset = Data.SelectionRange.Offset;
 					RunAction (ClipboardActions.Copy);
 					Reset (string.Empty);
@@ -422,11 +552,12 @@ namespace Mono.TextEditor.Vi
 			case State.Change:
 				//copied from delete action
 				if (((modifier & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0 
-				     && key == Gdk.Key.c))
+				     && unicodeKey == 'c'))
 				{
 					action = SelectionActions.LineActionFromMoveAction (CaretMoveActions.LineEnd);
+					lineAction = true;
 				} else {
-					action = ViActionMaps.GetNavCharAction ((char)unicodeKey);
+					action = ViActionMaps.GetEditObjectCharAction ((char)unicodeKey);
 					if (action == null)
 						action = ViActionMaps.GetDirectionKeyAction (key, modifier);
 					if (action != null)
@@ -434,8 +565,11 @@ namespace Mono.TextEditor.Vi
 				}
 				
 				if (action != null) {
-					RunActions (action, ClipboardActions.Cut);
-					Reset ("-- INSERT --");
+					if (lineAction)
+						RunActions (action, ClipboardActions.Cut, ViActions.NewLineAbove);
+					else
+						RunActions (action, ClipboardActions.Cut);
+					Status = "-- INSERT --";
 					state = State.Insert;
 					Caret.Mode = CaretMode.Insert;
 				} else {
@@ -501,7 +635,7 @@ namespace Mono.TextEditor.Vi
 					action = ViActionMaps.GetCommandCharAction ((char)unicodeKey);
 				}
 				if (action != null) {
-					RunAction (SelectionActions.FromMoveAction (action));
+					RunAction (ViActions.VisualSelectionFromMoveAction (action));
 					return;
 				}
 
@@ -540,15 +674,15 @@ namespace Mono.TextEditor.Vi
 					RunAction (SelectionActions.StartSelection);
 					int   roffset = Data.SelectionRange.Offset;
 					InsertCharacter ((char) unicodeKey);
-					Caret.Offset = roffset;
 					Reset (string.Empty);
+					Caret.Offset = roffset;
 				} else {
 					Reset ("Keystroke was not a character");
 				}
 				return;
 				
 			case State.Indent:
-				if (((modifier & (Gdk.ModifierType.ControlMask)) == 0 && ((char)unicodeKey) == '>'))
+				if (((modifier & (Gdk.ModifierType.ControlMask)) == 0 && unicodeKey == '>'))
 				{
 					RunAction (MiscActions.IndentSelection);
 					Reset ("");
@@ -598,6 +732,65 @@ namespace Mono.TextEditor.Vi
 				}
 				Reset ("Unknown command");
 				return;
+				
+			case State.Mark: {
+				char k = (char)unicodeKey;
+				ViMark mark = null;
+				if (!char.IsLetterOrDigit(k)) {
+					Reset ("Invalid Mark");
+					return;
+				}
+				if (marks.ContainsKey(k)) {
+					mark = marks [k];
+				} else {
+					mark = new ViMark(k);
+					marks [k] = mark;
+				}
+				RunAction(mark.SaveMark);
+				Reset("");
+				return;
+			}
+			
+			case State.NameMacro: {
+				char k = (char) unicodeKey;
+				if(!char.IsLetterOrDigit(k)) {
+					Reset("Invalid Macro Name");
+					return;
+				}
+				currentMacro = new ViMacro (k);
+				currentMacro.KeysPressed = new Queue<ViMacro.KeySet> ();
+				macros [k] = currentMacro;
+				Reset("");
+				return;
+			}
+			
+			case State.PlayMacro: {
+				char k = (char) unicodeKey;
+				if (macros.ContainsKey(k)) {
+					Reset ("");
+					ViMacro macroToPlay = macros [k];
+					foreach (ViMacro.KeySet keySet in macroToPlay.KeysPressed) {
+						HandleKeypress(keySet.Key, keySet.UnicodeKey, keySet.Modifiers);
+					}
+					/* Once all the keys have been played back, quickly exit. */
+					return;
+				} else {
+					Reset ("Invalid Macro Name");
+					return;
+				}
+			}
+			
+			case State.GoToMark: {
+				char k = (char)unicodeKey;
+				if (marks.ContainsKey(k)) {
+					RunAction(marks [k].LoadMark);
+					Reset ("");
+				} else {
+					Reset ("Unknown Mark");
+				}
+				return;
+			}
+				
 			case State.Fold:
 				if (((modifier & (Gdk.ModifierType.ControlMask)) == 0)) {
 					switch ((char)unicodeKey) {
@@ -663,8 +856,8 @@ namespace Mono.TextEditor.Vi
 				segment = Data.SelectionRange;
 			} else {
 				// Operate on current line
-				segment = Editor.Document.GetLine (Caret.Line);
-				line = Editor.Document.GetTextBetween (segment.Offset, segment.EndOffset);
+				segment = Data.Document.GetLine (Caret.Line);
+				line = Data.Document.GetTextBetween (segment.Offset, segment.EndOffset);
 			}
 
 			// Set regex options
@@ -678,7 +871,7 @@ namespace Mono.TextEditor.Vi
 
 			try {
 				string newline = Regex.Replace (line, match.Groups["pattern"].Value, replacement, options);
-				Editor.GetTextEditorData ().Replace (segment.Offset, line.Length, newline);
+				Data.Replace (segment.Offset, line.Length, newline);
 				if (Data.IsSomethingSelected)
 					Data.ClearSelection ();
 				lastPattern = match.Groups["pattern"].Value;
@@ -863,7 +1056,11 @@ namespace Mono.TextEditor.Vi
 			Indent,
 			Unindent,
 			G,
-			Fold
+			Fold,
+			Mark,
+			GoToMark,
+			NameMacro,
+			PlayMacro
 		}
 	}
 }
