@@ -74,20 +74,10 @@ namespace Mono.TextEditor
 			public static readonly Gdk.Atom RTF_ATOM = Gdk.Atom.Intern ("text/rtf", false);
 			public static readonly Gdk.Atom MD_ATOM  = Gdk.Atom.Intern ("text/monotext", false);
 			
-			TextEditorData data;
-			Selection selection;
-			
 			public CopyOperation ()	
 			{
 			}
 			
-			public CopyOperation (TextEditorData data, Selection selection)	
-			{
-				this.data = data;
-				this.selection = selection;
-				isBlockMode = data.SelectionMode == SelectionMode.Block;
-			}
-			
 			public void SetData (SelectionData selection_data, uint info)
 			{
 				if (selection_data == null)
@@ -322,25 +312,15 @@ namespace Mono.TextEditor
 				if (Copy != null)
 					Copy (copiedDocument != null ? copiedDocument.Text : null);
 			}
-			
-			public void ClipboardGetFuncLazy (Clipboard clipboard, SelectionData selection_data, uint info)
-			{
-				// data may have disposed
-				if (data.Document == null)
-					return;
-				CopyData (data, selection);
-				ClipboardGetFunc (clipboard, selection_data, info);
-			}
-			
+		
 			public delegate void CopyDelegate (string text);
 			public static event CopyDelegate Copy;
 		}
 		
 		public static void CopyToPrimary (TextEditorData data)
 		{
-			CopyOperation operation = new CopyOperation (data, data.MainSelection);
 			Clipboard clipboard = Clipboard.Get (CopyOperation.PRIMARYCLIPBOARD_ATOM);
-			clipboard.SetWithData ((Gtk.TargetEntry[])CopyOperation.targetList, operation.ClipboardGetFuncLazy, operation.ClipboardClearFunc);
+			clipboard.Text = data.SelectedText;
 		}
 		
 		public static void ClearPrimary ()
@@ -364,11 +344,13 @@ namespace Mono.TextEditor
 					string text = System.Text.Encoding.UTF8.GetString (selBytes, 1, selBytes.Length - 1);
 					bool pasteBlock = (selBytes[0] & 1) == 1;
 					bool pasteLine = (selBytes[0] & 2) == 2;
-
+					if (!pasteBlock && !pasteLine)
+						return;
+					
 					data.Document.BeginAtomicUndo ();
 					if (preserveSelection && data.IsSomethingSelected)
 						data.DeleteSelectedText ();
-
+					
 					data.Caret.PreserveSelection = true;
 					if (pasteBlock) {
 						string[] lines = text.Split ('\r');
@@ -416,36 +398,60 @@ namespace Mono.TextEditor
 			if (result < 0) {
 				clipboard.WaitIsTextAvailable ();
 				clipboard.RequestText (delegate(Clipboard clp, string text) {
+					if (string.IsNullOrEmpty (text))
+						return;
 					data.Document.BeginAtomicUndo ();
 					int caretPos = data.Caret.Offset;
-					ISegment selection = data.SelectionRange;
-					if (preserveSelection && data.IsSomethingSelected)
-						data.DeleteSelectedText ();
-
-					data.Caret.PreserveSelection = true;
-					//int oldLine = data.Caret.Line;
-					int textLength = data.Insert (insertionOffset, text);
-					result = textLength;
-
-					if (data.IsSomethingSelected && data.SelectionRange.Offset >= insertionOffset)
-						data.SelectionRange.Offset += textLength;
-					if (data.IsSomethingSelected && data.MainSelection.GetAnchorOffset (data) >= insertionOffset)
-						data.MainSelection.Anchor = data.Document.OffsetToLocation (data.MainSelection.GetAnchorOffset (data) + textLength);
-
-					data.Caret.PreserveSelection = false;
-					if (!preserveState) {
-						data.Caret.Offset = insertionOffset + textLength;
-					} else {
-						if (caretPos >= insertionOffset)
-							data.Caret.Offset = caretPos + textLength;
-						if (selection != null) {
-							int offset = selection.Offset;
-							if (offset >= insertionOffset)
-								offset += textLength;
-							data.SelectionRange = new Segment (offset, selection.Length);
+					if (data.IsSomethingSelected && data.MainSelection.SelectionMode == SelectionMode.Block) {
+						data.Caret.PreserveSelection = true;
+						if (!data.MainSelection.IsDirty) {
+							data.DeleteSelectedText (false);
+							data.MainSelection.IsDirty = true;
 						}
+						int textLength = 0;
+						int column = data.Caret.Column;
+						int minLine = data.MainSelection.MinLine;
+						int maxLine = data.MainSelection.MaxLine;
+						for (int lineNumber = minLine; lineNumber <= maxLine; lineNumber++) {
+							int offset = data.Document.GetLine (lineNumber).Offset + column;
+							textLength = data.Insert (offset, text);
+							data.PasteText (offset, text);
+						}
+						
+						data.Caret.Offset += textLength;
+						data.MainSelection.Anchor = new DocumentLocation (data.Caret.Line == minLine ? maxLine : minLine, data.Caret.Column - textLength);
+						data.MainSelection.Lead = new DocumentLocation (data.Caret.Line, data.Caret.Column);
+						data.Caret.PreserveSelection = false;
+						data.Document.CommitMultipleLineUpdate (data.MainSelection.MinLine, data.MainSelection.MaxLine);
+					} else {
+						ISegment selection = data.SelectionRange;
+						if (preserveSelection && data.IsSomethingSelected)
+							data.DeleteSelectedText ();
+						data.Caret.PreserveSelection = true;
+						//int oldLine = data.Caret.Line;
+						int textLength = data.Insert (insertionOffset, text);
+						result = textLength;
+	
+						if (data.IsSomethingSelected && data.SelectionRange.Offset >= insertionOffset)
+							data.SelectionRange.Offset += textLength;
+						if (data.IsSomethingSelected && data.MainSelection.GetAnchorOffset (data) >= insertionOffset)
+							data.MainSelection.Anchor = data.Document.OffsetToLocation (data.MainSelection.GetAnchorOffset (data) + textLength);
+						
+						data.Caret.PreserveSelection = false;
+						if (!preserveState) {
+							data.Caret.Offset += textLength;
+						} else {
+							if (caretPos >= insertionOffset)
+								data.Caret.Offset += textLength;
+							if (selection != null) {
+								int offset = selection.Offset;
+								if (offset >= insertionOffset)
+									offset += textLength;
+								data.SelectionRange = new Segment (offset, selection.Length);
+							}
+						}
+						data.PasteText (insertionOffset, text);
 					}
-					data.PasteText (insertionOffset, text);
 					data.Document.EndAtomicUndo ();
 				});
 			}
@@ -463,12 +469,10 @@ namespace Mono.TextEditor
 			if (!data.CanEditSelection)
 				return;
 			LineSegment line = data.Document.GetLine (data.Caret.Line);
-			int offset = data.Caret.Offset;
 			if (data.Caret.Column > line.EditableLength) {
 				string text = data.GetVirtualSpaces (data.Caret.Line, data.Caret.Column);
 				int textLength = data.Insert (data.Caret.Offset, text);
-				offset += textLength;
-				data.Caret.Offset = offset;
+				data.Caret.Offset += textLength;
 			}
 			PasteFrom (Clipboard.Get (CopyOperation.CLIPBOARD_ATOM), data, true, data.IsSomethingSelected ? data.SelectionRange.Offset : data.Caret.Offset);
 		}
