@@ -37,7 +37,7 @@ namespace MonoDevelop.Core.Serialization
 	{
 		Hashtable properties = new Hashtable ();
 		List<ItemProperty> sortedPoperties = new List<ItemProperty> ();
-		ArrayList subtypes;
+		List<ClassDataType> subtypes;
 		Type fallbackType;
 		
 		public ClassDataType (Type propType): base (propType)
@@ -106,7 +106,10 @@ namespace MonoDevelop.Core.Serialization
 		
 		void AddProperty (object member, string name, Type memberType)
 		{
-			object[] ats = Context.AttributeProvider.GetCustomAttributes (member, typeof(Attribute), true);
+			// Using inherit=false because if a base class already has an ItemProperty applied to the property
+			// then that property will already have been added while copying props from the base class.
+			
+			object[] ats = Context.AttributeProvider.GetCustomAttributes (member, typeof(Attribute), false);
 			
 			ItemPropertyAttribute at = FindPropertyAttribute (ats, "");
 			if (at == null)
@@ -118,6 +121,8 @@ namespace MonoDevelop.Core.Serialization
 			prop.DefaultValue = at.DefaultValue;
 			prop.IsExternal = at.IsExternal;
 			prop.SkipEmpty = at.SkipEmpty;
+			prop.ReadOnly = at.ReadOnly;
+			prop.WriteOnly = at.WriteOnly;
 
 			if (prop.ExpandedCollection) {
 				ICollectionHandler handler = Context.GetCollectionHandler (memberType);
@@ -169,7 +174,7 @@ namespace MonoDevelop.Core.Serialization
 		private void AddSubtype (ClassDataType subtype)
 		{
 			if (subtypes == null)
-				subtypes = new ArrayList ();
+				subtypes = new List<ClassDataType> ();
 			subtypes.Add (subtype); 
 		}
 
@@ -271,7 +276,13 @@ namespace MonoDevelop.Core.Serialization
 				}
 				if (ctype == null) {
 					DataType subtype = Context.GetConfigurationDataType (obj.GetType ());
-					DataItem it = (DataItem) subtype.Serialize (serCtx, mapData, obj);
+					DataNode n = subtype.Serialize (serCtx, mapData, obj);
+					DataItem it = n as DataItem;
+					if (it == null) {
+						it = new DataItem ();
+						n.Name = "Value";
+						it.ItemData.Add (n);
+					}
 					it.ItemData.Add (new DataValue ("ctype", subtype.Name));
 					it.Name = Name;
 					return it;
@@ -372,7 +383,13 @@ namespace MonoDevelop.Core.Serialization
 				}
 				
 				if (stype != null) {
-					object sobj = stype.Deserialize (serCtx, mapData, data);
+					DataNode desData = data;
+					if (stype.IsSimpleType) {
+						desData = item.ItemData ["Value"];
+						if (desData == null)
+							throw new InvalidOperationException ("Value node not found");
+					}
+					object sobj = stype.Deserialize (serCtx, mapData, desData);
 					
 					// Store the original data type, so it can be serialized back
 					if (isFallbackType && sobj is IExtendedDataItem) {
@@ -528,7 +545,7 @@ namespace MonoDevelop.Core.Serialization
 		{
 			isFallbackType = false;
 			if (subtypes != null) {
-				foreach (ClassDataType stype in subtypes) {
+				foreach (ClassDataType stype in new List<ClassDataType> (subtypes)) {
 					if (stype.Name == name) 
 						return stype;
 						
@@ -540,11 +557,21 @@ namespace MonoDevelop.Core.Serialization
 					}
 				}
 			}
+			
+			// Try a direct type name query
+			DataType dt = Context.GetConfigurationDataType (name);
+			if (dt != null && ValueType.IsAssignableFrom (dt.ValueType))
+				return dt;
+			
+			// The type could not be found. Use the fallback type if available.
+			
+			// Properties can specify a specific fallback type. It is provided in the custom map data.
 			if (mapData != null) {
 				isFallbackType = true;
 				return Context.GetConfigurationDataType ((Type)mapData);
 			}
 			
+			// A type can have a default fallback type.
 			if (fallbackType != null) {
 				isFallbackType = true;
 				return Context.GetConfigurationDataType (fallbackType);
