@@ -52,7 +52,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 		InExpression
 	}
 	
-	public interface ITemplateWidget 
+	public interface ICodeTemplateContextProvider
 	{
 		CodeTemplateContext GetCodeTemplateContext ();
 	}
@@ -99,7 +99,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 			set;
 		}
 		
-		public string Icon {
+		public IconId Icon {
 			get {
 				return Code.Contains ("$selected$") ? "md-template-surroundwith" : "md-template";
 			}
@@ -215,7 +215,9 @@ namespace MonoDevelop.Ide.CodeTemplates
 				string name = match.Groups[1].Value;
 				sb.Append (code.Substring (lastOffset, match.Index - lastOffset));
 				lastOffset = match.Index + match.Length;
-				if (name == "end") {
+				if (string.IsNullOrEmpty (name)) { // $$ is interpreted as $
+					sb.Append ("$");
+				} else if (name == "end") {
 					result.CaretEndOffset = sb.Length;
 				} else if (name == "selected") {
 					if (!string.IsNullOrEmpty (context.SelectedText)) {
@@ -272,13 +274,23 @@ namespace MonoDevelop.Ide.CodeTemplates
 		}
 
 		
-		public string IndentCode (string code, string indent)
+		public string IndentCode (string code, string eol, string indent)
 		{
 			StringBuilder result = new StringBuilder ();
 			for (int i = 0; i < code.Length; i++) {
-				result.Append (code[i]);
-				if (code[i] == '\n')
+				switch (code[i]) {
+				case '\r':
+					if (i + 1 < code.Length && code[i + 1] == '\n')
+						i++;
+					goto case '\n';
+				case '\n':
+					result.Append (eol);
 					result.Append (indent);
+					break;
+				default:
+					result.Append (code[i]);
+					break;
+				}
 			}
 			return result.ToString ();
 		}
@@ -342,14 +354,26 @@ namespace MonoDevelop.Ide.CodeTemplates
 			return result.ToString ();
 		}
 		
-		public TemplateResult InsertTemplate (MonoDevelop.Ide.Gui.Document document)
+		public void Insert (MonoDevelop.Ide.Gui.Document document)
 		{
-			ProjectDom dom = ProjectDomService.GetProjectDom (document.Project) ?? ProjectDomService.GetFileDom (document.FileName);
+			var handler = document.GetContent<ICodeTemplateHandler> ();
+			if (handler != null) {
+				handler.InsertTemplate (this, document);
+			} else {
+				InsertTemplateContents (document);
+			}	
+		}
+		
+		/// <summary>
+		/// Don't use this unless you're implementing ICodeTemplateWidget. Use Insert instead.
+		/// </summary>
+		public TemplateResult InsertTemplateContents (MonoDevelop.Ide.Gui.Document document)
+		{
+			ProjectDom dom = document.Dom;
 			ParsedDocument doc = document.ParsedDocument ?? MonoDevelop.Projects.Dom.Parser.ProjectDomService.GetParsedDocument (dom, document.FileName);
 			MonoDevelop.Ide.Gui.TextEditor editor = document.TextEditor;
 
-			Mono.TextEditor.ITextEditorDataProvider provider = document.GetContent<Mono.TextEditor.ITextEditorDataProvider> ();
-			Mono.TextEditor.TextEditorData data = provider.GetTextEditorData ();
+			Mono.TextEditor.TextEditorData data = document.TextEditorData;
 
 			int offset = editor.CursorPosition;
 			int line, col;
@@ -363,7 +387,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 				ParsedDocument = doc,
 				InsertPosition = new DomLocation (line, col),
 				LineIndent = GetIndent (editor, line, 0),
-				TemplateCode = IndentCode (Code, GetIndent (editor, line, 0))
+				TemplateCode = IndentCode (Code, document.TextEditorData.EolMarker, GetIndent (editor, line, 0))
 			};
 
 			if (data.IsSomethingSelected) {
@@ -387,12 +411,12 @@ namespace MonoDevelop.Ide.CodeTemplates
 			
 			TemplateResult template = FillVariables (context);
 			template.InsertPosition = offset;
-			editor.InsertText (offset, template.Code);
+			document.TextEditorData.Insert (offset, template.Code);
 			
 			if (template.CaretEndOffset >= 0) {
-				editor.CursorPosition = offset + template.CaretEndOffset; 
+				document.TextEditorData.Caret.Offset = offset + template.CaretEndOffset; 
 			} else {
-				editor.CursorPosition = offset + template.Code.Length; 
+				document.TextEditorData.Caret.Offset= offset + template.Code.Length; 
 			}
 			return template;
 		}
