@@ -26,6 +26,7 @@
 //
 
 using System;
+using MonoDevelop.Components.Commands;
 
 namespace MonoDevelop.Ide.Commands
 {
@@ -84,5 +85,18 @@ namespace MonoDevelop.Ide.Commands
 		SelectionMoveNextSubword,
 		DeletePrevSubword,
 		DeleteNextSubword,
+		TransposeCharacters,
+		RecenterEditor,
+		
+		ToggleCompletionSuggestionMode,
+		ToggleBlockSelectionMode
+	}
+	
+	public class ToggleCompletionSuggestionModeHandler : CommandHandler
+	{
+		protected override void Run ()
+		{
+			MonoDevelop.Ide.CodeCompletion.CompletionWindowManager.ForceSuggestionMode = !MonoDevelop.Ide.CodeCompletion.CompletionWindowManager.ForceSuggestionMode;
+		}
 	}
 }
