@@ -28,20 +28,13 @@
 
 
 using System;
-using System.Collections;
 using System.Diagnostics;
 using System.IO;
-using Gtk;
 
 using MonoDevelop.Core;
-using Mono.Addins;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Core.Gui.Dialogs;
 using MonoDevelop.Ide.Gui.Content;
-using MonoDevelop.Components;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Ide.Commands;
-using MonoDevelop.Ide.Gui.Dialogs;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -112,7 +105,7 @@ namespace MonoDevelop.Ide.Gui
 		[CommandHandler (FileCommands.ReloadFile)]
 		protected void OnReloadFile ()
 		{
-			if (MessageService.GenericAlert (MonoDevelop.Core.Gui.Stock.Warning,
+			if (MessageService.GenericAlert (MonoDevelop.Ide.Gui.Stock.Warning,
 			                                GettextCatalog.GetString ("Revert unsaved changes to document '{0}'?", Path.GetFileName (doc.Name)),
 			                                GettextCatalog.GetString ("All changes made to the document will be permanently lost."), 0, AlertButton.Cancel, AlertButton.Revert) == AlertButton.Revert)
 				doc.Reload ();
@@ -130,7 +123,7 @@ namespace MonoDevelop.Ide.Gui
 		[CommandHandler (EditCommands.Undo)]
 		protected void OnUndo ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
+			IUndoHandler editable = GetContent <IUndoHandler> ();
 			if (editable != null)
 				editable.Undo();
 		}
@@ -138,14 +131,14 @@ namespace MonoDevelop.Ide.Gui
 		[CommandUpdateHandler (EditCommands.Undo)]
 		protected void OnUpdateUndo (CommandInfo info)
 		{
-			IEditableTextBuffer textBuffer = GetContent <IEditableTextBuffer> ();
+			IUndoHandler textBuffer = GetContent <IUndoHandler> ();
 			info.Enabled = textBuffer != null && textBuffer.EnableUndo;
 		}
 		
 		[CommandHandler (EditCommands.Redo)]
 		protected void OnRedo ()
 		{
-			IEditableTextBuffer editable = GetContent <IEditableTextBuffer> ();
+			IUndoHandler editable = GetContent <IUndoHandler> ();
 			if (editable != null) {
 				editable.Redo();
 			}
@@ -154,7 +147,7 @@ namespace MonoDevelop.Ide.Gui
 		[CommandUpdateHandler (EditCommands.Redo)]
 		protected void OnUpdateRedo (CommandInfo info)
 		{
-			IEditableTextBuffer textBuffer = GetContent <IEditableTextBuffer> ();
+			IUndoHandler textBuffer = GetContent <IUndoHandler> ();
 			info.Enabled = textBuffer != null && textBuffer.EnableRedo;
 		}
 		
@@ -170,8 +163,8 @@ namespace MonoDevelop.Ide.Gui
 		protected void OnUpdateCut (CommandInfo info)
 		{
 			IClipboardHandler handler = GetContent <IClipboardHandler> ();
-			if (handler != null)
-				info.Enabled = handler.EnableCut;
+			if (handler != null && handler.EnableCut)
+				info.Enabled = true;
 			else
 				info.Bypass = true;
 		}
@@ -188,8 +181,8 @@ namespace MonoDevelop.Ide.Gui
 		protected void OnUpdateCopy (CommandInfo info)
 		{
 			IClipboardHandler handler = GetContent <IClipboardHandler> ();
-			if (handler != null)
-				info.Enabled = handler.EnableCopy;
+			if (handler != null && handler.EnableCopy)
+				info.Enabled = true;
 			else
 				info.Bypass = true;
 		}
@@ -206,8 +199,8 @@ namespace MonoDevelop.Ide.Gui
 		protected void OnUpdatePaste (CommandInfo info)
 		{
 			IClipboardHandler handler = GetContent <IClipboardHandler> ();
-			if (handler != null)
-				info.Enabled = handler.EnablePaste;
+			if (handler != null && handler.EnablePaste)
+				info.Enabled = true;
 			else
 				info.Bypass = true;
 		}
@@ -280,30 +273,28 @@ namespace MonoDevelop.Ide.Gui
 				int cursor_pos = buffer.CursorPosition;
 				int line, column;
 				buffer.GetLineColumnFromPosition (buffer.CursorPosition, out line, out column);
-
-				int start_pos = buffer.GetPositionFromLineColumn (line, 0)+1;
+				if (line >= doc.TextEditor.LineCount)
+					return;
+				int start_pos = buffer.GetPositionFromLineColumn (line, 0) + 1;
 				
 				int line_len = doc.TextEditor.GetLineLength (line);
-				int next_line_len = doc.TextEditor.GetLineLength (line+1);
+				int next_line_len = doc.TextEditor.GetLineLength (line + 1);
 				
-				if (next_line_len <= 0) {
+				if (next_line_len < 0) 
 					return;
-				}
 				
 				int end_pos = start_pos + line_len + next_line_len + 1;
 				
 				string curr_line = doc.TextEditor.GetLineText (line);
-				string next_line = doc.TextEditor.GetLineText (line+1);
+				string next_line = doc.TextEditor.GetLineText (line + 1);
 
 				string new_text = curr_line;
 				string next_line_trimmed = next_line.TrimStart ('\n', '\r', '\t', ' ');
-				if (next_line_trimmed != null &&
-				    next_line_trimmed != String.Empty) {
+				if (!string.IsNullOrEmpty (next_line_trimmed)) 
 					new_text += " " + next_line_trimmed;
-				}
 				
 				buffer.BeginAtomicUndo ();
-				buffer.DeleteText (start_pos, end_pos-start_pos);
+				buffer.DeleteText (start_pos, end_pos - start_pos);
 				buffer.InsertText (start_pos, new_text);
 				buffer.EndAtomicUndo ();
 
@@ -452,12 +443,6 @@ namespace MonoDevelop.Ide.Gui
 			doc.TextEditor.CursorPosition = doc.TextEditor.TextLength;
 		}
 		
-		[CommandHandler (TextEditorCommands.GotoMatchingBrace)]
-		protected void OnGotoMatchingBrace ()
-		{
-			doc.TextEditor.GotoMatchingBrace ();
-		}
-		
 		[CommandHandler (TextEditorCommands.DeleteLine)]
 		protected void OnDeleteLine ()
 		{
@@ -481,85 +466,87 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		[CommandHandler (TextEditorCommands.MoveBlockUp)]
-		protected void OnMoveBlockUp ()
+		
+		struct RemoveInfo
 		{
-			int lineStart = -1, lineEnd = -1, colStart = -1, colEnd = -1;
-			doc.TextEditor.GetLineColumnFromPosition (doc.TextEditor.SelectionStartPosition, out lineStart, out colStart);
-			doc.TextEditor.GetLineColumnFromPosition (doc.TextEditor.SelectionEndPosition, out lineEnd, out colEnd);
-			
-			// Full line selection behaves oddly, in that the end position will be reported as
-			// the first column of the next line.  We don't want that behavior.
-			if(lineStart != lineEnd && colEnd == 1)
-				lineEnd--;
-			
-			if(lineStart == 1)
-				return;
-			
-			// We don't want to do partial lines.  If any part of a line is selected,
-			// we move the entire line.
+			public int Position { get; set; }
+			public int Length { get; set; }
 
-			doc.TextEditor.BeginAtomicUndo ();
-			int oldSelStart = doc.TextEditor.SelectionStartPosition;
-			int oldSelEnd = doc.TextEditor.SelectionEndPosition;
+			public static readonly RemoveInfo Empty = new RemoveInfo (-1, -1);
 			
-			int startPos = doc.TextEditor.GetPositionFromLineColumn (lineStart, 1);
-			int endPos = doc.TextEditor.GetPositionFromLineColumn (lineEnd, doc.TextEditor.GetLineLength(lineEnd) + 1) + 1;  // Include \n
+			public bool IsEmpty {
+				get {
+					return Length <= 0;
+				}
+			}
 			
-			string text = doc.TextEditor.GetText(startPos, endPos);
-			doc.TextEditor.DeleteText(startPos, endPos - startPos);
-			int newStartPos = doc.TextEditor.GetPositionFromLineColumn(lineStart - 1, 1);
-			doc.TextEditor.InsertText(newStartPos, text);
-			
-			// Now we either reset the selection or the cursor, depending
-			// on what the case was when we started.
-			int selStart = newStartPos + colStart - 1;
-			int selEnd = selStart + (oldSelEnd - oldSelStart);
-			doc.TextEditor.Select(selStart, selEnd);
+			RemoveInfo (int position, int length): this ()
+			{
+				Position = position;
+				Length = length;
+			}
 
-			doc.TextEditor.EndAtomicUndo ();
+			public static bool IsWhiteSpace (char ch) 
+			{
+				return ch == ' ' || ch == '\t' || ch == '\v';
+			}
+			
+			public static RemoveInfo GetRemoveInfo (Mono.TextEditor.Document document, ref int pos)
+			{
+				int len = 0;
+				while (pos > 0 && IsWhiteSpace (document.GetCharAt (pos))) {
+					--pos;
+					++len;
+				}
+				if (len > 0) {
+					pos++;
+					return new RemoveInfo (pos, len);
+				}
+				return Empty;
+			}
+
+			public override string ToString ()
+			{
+				return string.Format ("[RemoveInfo: Position={0}, Length={1}]", Position, Length);
+			}
 		}
 		
-		[CommandHandler (TextEditorCommands.MoveBlockDown)]
-		protected void OnMoveBlockDown ()
+		[CommandHandler (EditCommands.RemoveTrailingWhiteSpaces)]
+		public void OnRemoveTrailingWhiteSpaces ()
 		{
-			int lineStart = -1, lineEnd = -1, colStart = -1, colEnd = -1;
-			doc.TextEditor.GetLineColumnFromPosition (doc.TextEditor.SelectionStartPosition, out lineStart, out colStart);
-			doc.TextEditor.GetLineColumnFromPosition (doc.TextEditor.SelectionEndPosition, out lineEnd, out colEnd);
-			
-			// Full line selection behaves oddly, in that the end position will be reported as
-			// the first column of the next line.  We don't want that behavior.
-			if(lineStart != lineEnd && colEnd == 1)
-				lineEnd--;
-			
-			if(doc.TextEditor.GetPositionFromLineColumn (lineEnd + 1, 1) == -1)
+			Mono.TextEditor.TextEditorData data = doc.TextEditorData;
+			if (data == null)
 				return;
 			
-			// We don't want to do partial lines.  If any part of a line is selected,
-			// we move the entire line.
-
-			doc.TextEditor.BeginAtomicUndo ();
-			int oldSelStart = doc.TextEditor.SelectionStartPosition;
-			int oldSelEnd = doc.TextEditor.SelectionEndPosition;
-
-			int startPos = doc.TextEditor.GetPositionFromLineColumn (lineStart, 1);
-			int endPos = doc.TextEditor.GetPositionFromLineColumn (lineEnd, doc.TextEditor.GetLineLength(lineEnd) + 1) + 1;  // Include \n
-
-			// If the next line is the last line, don't add 1 for the \n or we'll end up adding an extra line.
-			if (doc.TextEditor.GetPositionFromLineColumn (lineEnd + 2, 1) == -1)
-				endPos--;
+			System.Collections.Generic.List<RemoveInfo> removeList = new System.Collections.Generic.List<RemoveInfo> ();
+			int pos = data.Document.Length - 1;
+			RemoveInfo removeInfo = RemoveInfo.GetRemoveInfo (data.Document, ref pos);
+			if (!removeInfo.IsEmpty)
+				removeList.Add (removeInfo);
 			
-			string text = doc.TextEditor.GetText(startPos, endPos);
-			doc.TextEditor.DeleteText(startPos, endPos - startPos);
-			int newStartPos = doc.TextEditor.GetPositionFromLineColumn(lineStart + 1, 1);
-			doc.TextEditor.InsertText(newStartPos, text);
+			while (pos >= 0) {
+				char ch = data.Document.GetCharAt (pos);
+				if (ch == '\n' || ch == '\r') {
+					if (RemoveInfo.IsWhiteSpace (data.Document.GetCharAt (pos - 1))) {
+						--pos;
+						removeInfo = RemoveInfo.GetRemoveInfo (data.Document, ref pos);
+						if (!removeInfo.IsEmpty)
+							removeList.Add (removeInfo);
+					}
+				}
+				--pos;
+			}
 			
-			// Now we either reset the selection or the cursor, depending
-			// on what the case was when we started.
-			int selStart = newStartPos + colStart - 1;
-			int selEnd = selStart + (oldSelEnd - oldSelStart);
-			doc.TextEditor.Select(selStart, selEnd);
-			doc.TextEditor.EndAtomicUndo ();
+			data.Document.BeginAtomicUndo ();
+			removeList.ForEach (info => ((Mono.TextEditor.IBuffer)data.Document).Remove (info.Position, info.Length));
+			data.Caret.Offset = Math.Min (data.Caret.Offset, data.Document.Length - 1);
+			data.Document.EndAtomicUndo ();
+		}
+		
+		[CommandUpdateHandler (EditCommands.RemoveTrailingWhiteSpaces)]
+		protected void OnRemoveTrailingWhiteSpaces (CommandInfo info)
+		{
+			info.Enabled = GetContent <IEditableTextBuffer> () != null;
 		}
 		
 		#region Folding
