@@ -36,7 +36,6 @@ using MonoDevelop.Core;
 using MonoDevelop.Projects;
 using MonoDevelop.Ide;
 using MonoDevelop.Ide.Gui;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Projects.Dom;
@@ -478,10 +477,10 @@ namespace MonoDevelop.Ide.Tasks
 			if (task != null && ! String.IsNullOrEmpty (task.FileName)) {
 				Document doc = IdeApp.Workbench.OpenDocument (task.FileName, Math.Max (1, task.Line), Math.Max (1, task.Column), true);
 				if (doc != null && doc.HasProject && doc.Project is DotNetProject) {
-					IDotNetLanguageBinding binding = ((DotNetProject)doc.Project).LanguageBinding;
-					if (! String.IsNullOrEmpty (binding.SingleLineCommentTag)) {
+					string[] commentTags = TextEditor.GetCommentTags (doc.FileName);
+					if (commentTags != null && commentTags.Length == 1) {
 						string line = doc.TextEditor.GetLineText (task.Line);
-						int index = line.IndexOf (binding.SingleLineCommentTag);
+						int index = line.IndexOf (commentTags[0]);
 						if (index != -1) {
 							doc.TextEditor.JumpTo (task.Line, task.Column);
 							line = line.Substring (0, index);
@@ -601,7 +600,7 @@ namespace MonoDevelop.Ide.Tasks
 			} 
 		}
 		
-		ToolItem[] ITaskListView.ToolBarItems {
+		Widget[] ITaskListView.ToolBarItems {
 			get { return null; } 
 		}
 		#endregion
