@@ -57,7 +57,7 @@ namespace MonoDevelop.Ide.Templates
 				AddStandardHeader = true;
 		}
 		
-		public override string CreateContent (Project project, Dictionary<string,string> tags, string language)
+		public override string CreateContent (Project project, Dictionary<string, string> tags, string language)
 		{
 			if (language == null || language == "")
 				throw new InvalidOperationException ("Language not defined in CodeDom based template.");
@@ -67,7 +67,7 @@ namespace MonoDevelop.Ide.Templates
 			CodeDomProvider provider = null;
 			if (binding != null)
 				provider = binding.GetCodeDomProvider ();
-				
+			
 			if (provider == null)
 				throw new InvalidOperationException ("The language '" + language + "' does not have support for CodeDom.");
 
@@ -77,23 +77,56 @@ namespace MonoDevelop.Ide.Templates
 			foreach (CodeNamespace cns in cu.Namespaces)
 				cns.Name = StripImplicitNamespace (project, tags, cns.Name);
 			
-			CodeGeneratorOptions options = new CodeGeneratorOptions();
+			CodeGeneratorOptions options = new CodeGeneratorOptions ();
 			options.IndentString = TextEditorProperties.IndentString;
 			options.BracingStyle = "C";
 			
 			StringWriter sw = new StringWriter ();
 			provider.GenerateCodeFromCompileUnit (cu, sw, options);
-			sw.Close();
+			sw.Close ();
 			
-			string txt = sw.ToString ();
-			int i = txt.IndexOf ("</autogenerated>");
-			if (i == -1) return txt;
-			i = txt.IndexOf ('\n', i);
-			if (i == -1) return txt;
-			i = txt.IndexOf ('\n', i + 1);
-			if (i == -1) return txt;
+			return StripHeaderAndBlankLines (sw.ToString (), provider);
+		}
+		
+		static string StripHeaderAndBlankLines (string text, CodeDomProvider provider)
+		{
+			Mono.TextEditor.Document doc = new Mono.TextEditor.Document ();
+			doc.Text = text;
+			int realStartLine = 0;
+			for (int i = 0; i < doc.LineCount; i++) {
+				string lineText = doc.GetTextAt (doc.GetLine (i));
+				// Microsoft.NET generates "auto-generated" tags where Mono generates "autogenerated" tags.
+				if (lineText.Contains ("</autogenerated>") || lineText.Contains ("</auto-generated>")) {
+					realStartLine = i + 2;
+					break;
+				}
+			}
 			
-			return txt.Substring (i+1);		
+			// The Mono provider inserts additional blank lines, so strip them out
+			// But blank lines might actually be significant in other languages.
+			// We reformat the C# generated output to the user's coding style anyway, but the reformatter preserves blank lines
+			if (provider is Microsoft.CSharp.CSharpCodeProvider) {
+				for (int i = 0; i < doc.LineCount; i++) {
+					Mono.TextEditor.LineSegment line = doc.GetLine (i);
+					if (IsBlankLine (doc, line) && line.Length > 0) {
+						((Mono.TextEditor.IBuffer)doc).Remove (line.Offset, line.Length);
+						i--;
+						continue;
+					}
+				}
+			}
+			
+			int offset = doc.GetLine (realStartLine).Offset;
+			return doc.GetTextAt (offset, doc.Length - offset);
+		}
+
+		static bool IsBlankLine (Mono.TextEditor.Document doc, Mono.TextEditor.LineSegment line)
+		{
+			for (int i = 0; i < line.EditableLength; i++) {
+				if (!Char.IsWhiteSpace (doc.GetCharAt (line.Offset + i)))
+					return false;
+			}
+			return true;
 		}
 		
 		internal static string StripImplicitNamespace (Project project, Dictionary<string,string> tags, string ns)
